###############################################################################
# Name: _dirmon.py                                                            #
# Purpose: Directory monitor object.                                          #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2011 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""
Editra Business Model Library: DirectoryMonitor


"""

__author__ = "Cody Precord <cprecord@editra.org>"
__cvsid__ = "$Id: _dirmon.py 73166 2012-12-12 04:31:53Z CJP $"
__revision__ = "$Revision: 73166 $"

__all__ = ['DirectoryMonitor',]

#-----------------------------------------------------------------------------#
# Imports
import wx
import os
import time
import threading

# Local imports
import fileutil

#-----------------------------------------------------------------------------#

class DirectoryMonitor(object):
    """Object to manage monitoring file system changes"""
    def __init__(self, checkFreq=1000.0):
        """@keyword checkFreq: check frequency in milliseconds"""
        super(DirectoryMonitor, self).__init__()

        # Attributes
        self._watcher = WatcherThread(self._ThreadNotifier, checkFreq=checkFreq)
        self._callbacks = list()
        self._cbackLock = threading.Lock()
        self._running = False

    def __del__(self):
        if self._running:
            self._watcher.Shutdown()
            self._watcher.join()

    def _ThreadNotifier(self, added, deleted, modified):
        """Notifier callback from background L{WatcherThread}
        to call notifiers on main thread.
        @note: this method is invoked from a background thread and
               is not safe to make direct UI calls from.

        """
        with self._cbackLock:
            for cback in self._callbacks:
                wx.CallAfter(cback, added, deleted, modified)

    #---- Properties ----#

    # Is the monitor currently watching any directories
    Monitoring = property(lambda self: self._running)
    Frequency = property(lambda self: self._watcher.GetFrequency(),
                         lambda self, freq: self._watcher.SetFrequency(freq))

    #---- End Properties ----#

    def AddDirectory(self, dname):
        """Add a directory to the monitor
        @param dname: directory path
        @return: bool - True if added, False if failed to add

        """
        return self._watcher.AddWatchDirectory(dname)

    def SubscribeCallback(self, callback):
        """Subscribe a callback method to be called when changes are
        detected in one of the watched directories.
        @param callback: callable([added,], [deleted,], [modified,])

        """
        with self._cbackLock:
            if callback not in self._callbacks:
                self._callbacks.append(callback)

    def UnsubscribeCallback(self, callback):
        """Remove a callback method from the monitor"""
        with self._cbackLock:
            if callback in self._callbacks:
                self._callbacks.remove(callback)

    def RemoveDirectory(self, dname):
        """Remove a directory from the watch list
        @param dname: directory path

        """
        self._watcher.RemoveWatchDirectory(dname)

    def StartMonitoring(self):
        """Start monitoring the directories in the watch list and
        notifying target of changes.

        """
        self._running = True
        self._watcher.start()

    def Suspend(self, pause=True):
        """Suspend background processing
        @keyword pause: True (suspend) False (resume)

        """
        if pause:
            self._watcher.Suspend()
        else:
            self._watcher.Continue()

    def Refresh(self, paths=None):
        """Force a recheck of the monitored directories. This method
        is useful for doing manual control of the refresh cycle. It is
        ignored and does nothing when WatcherThread is set up for automatic
        refresh cycles.
        @keyword paths: specific paths to refresh or None for all.

        """
        self._watcher.Refresh(paths)

#-----------------------------------------------------------------------------#
    
class WatcherThread(threading.Thread):
    """Background thread to monitor a directory"""
    def __init__(self, notifier, checkFreq=1000.0):
        """Create the WatcherThread. Provide a callback notifier method
        that will be called when changes are detected in the directory.
        The notifier will be called in the context of this thread. Notifier
        will be called with three lists of ebmlib.File objects to indicate
        the changes that have occurred.
        @param notifier: callable([added,], [deleted,], [modified,])
        @keyword checkFreq: check frequency in milliseconds. If value is set
                            to zero or less update checks must be manually
                            controlled via the Refresh interface.

        """
        super(WatcherThread, self).__init__()

        # Attributes
        assert callable(notifier)
        self._notifier = notifier
        self._dirs = list() # Directories being monitored
        self._refreshDirs = None

        self._freq = checkFreq # Monitoring frequency in milliseconds
        self._continue = True
        self._changePending = False
        self._lock = threading.Lock()
        self._suspend = False
        self._suspendcond = threading.Condition()
        self._listEmptyCond = threading.Condition()
        self._refreshCond = threading.Condition()

    def run(self):
        """Run the watcher"""
        while self._continue:
            deleted = list()
            added = list()
            modified = list()

            # Watch is empty so wait on things to monitor before continuing
            if not self._dirs:
                with self._listEmptyCond:
                    self._listEmptyCond.wait()

            # Suspend processing if requested
            if self._suspend:
                with self._suspendcond:
                    self._suspendcond.wait()

            with self._lock:
                for dobj in self._PendingRefresh:
                    if not self._continue:
                        return
                    elif self._changePending:
                        break

                    # Check if a watched directory has been deleted
                    if not os.path.exists(dobj.Path):
                        deleted.append(dobj)
                        self._dirs.remove(dobj)
                        continue

                    snapshot = fileutil.GetDirectoryObject(dobj.Path, 
                                                           False, True)

                    # Check for deletions
                    dobjFiles = dobj.Files # optimization
                    dobjIndex = dobjFiles.index # optimization
                    snapFiles = snapshot.Files # optimization
                    for tobj in dobjFiles:
                        if not self._continue:
                            return
                        elif self._changePending:
                            break
                        if tobj not in snapFiles:
                            deleted.append(tobj)
                            dobjFiles.remove(tobj)

                    # Check for additions and modifications
                    for tobj in snapFiles:
                        if not self._continue:
                            return
                        elif self._changePending:
                            break
                        if tobj not in dobjFiles:
                            # new object was added
                            added.append(tobj)
                            dobjFiles.append(tobj)
                        else:
                            idx = dobjIndex(tobj)
                            existing = dobjFiles[idx]
                            # object was modified
                            if existing.ModTime < tobj.ModTime:
                                modified.append(tobj)
                                existing.ModTime = tobj.ModTime

            # Call Notifier if anything changed
            if any((added, deleted, modified)):
                self._notifier(added, deleted, modified)

            # Wait till next check
            if self._freq > 0:
                # Automatic updates
                time.sleep(self._freq / 1000.0)
            else:
                # Manually controlled updates
                with self._refreshCond:
                    self._refreshDirs = None
                    self._refreshCond.wait()

    #---- Implementation ----#

    @property
    def _PendingRefresh(self):
        """Get the list of directories pending refresh"""
        if self._refreshDirs is None:
            return self._dirs
        else:
            return self._refreshDirs

    def AddWatchDirectory(self, dpath):
        """Add a directory to the watch list
        @param dpath: directory path (unicode)
        @return: bool - True means watch was added, False means unable to list directory

        """
        assert os.path.isdir(dpath)
        dobj = fileutil.Directory(dpath)
        self._changePending = True
        with self._lock:
            if dobj not in self._dirs and os.access(dobj.Path, os.R_OK):
                # Get current snapshot of the directory
                try:
                    dobj = fileutil.GetDirectoryObject(dpath, False, True)
                except OSError:
                    self._changePending = False
                    return False
                self._dirs.append(dobj)
                with self._listEmptyCond:
                    self._listEmptyCond.notify()
        self._changePending = False
        return True

    def RemoveWatchDirectory(self, dpath):
        """Remove a directory from the watch
        @param dpath: directory path to remove (unicode)

        """
        dobj = fileutil.Directory(dpath)
        self._changePending = True
        with self._lock:
            if dobj in self._dirs:
                self._dirs.remove(dobj)
            # Also remove any subpaths of dpath
            toremove = list()
            for d in self._dirs:
                if fileutil.IsSubPath(d.Path, dpath):
                    toremove.append(d)
            for todel in toremove:
                self._dirs.remove(todel)
        self._changePending = False

    def GetFrequency(self):
        """Get the update frequency
        @return: int (milliseconds)

        """
        return self._freq

    def SetFrequency(self, milli):
        """Set the update frequency
        @param milli: int (milliseconds)

        """
        self._freq = float(milli)

    def Refresh(self, paths=None):
        """Recheck the monitored directories
        only useful when manually controlling refresh cycle of the monitor.
        @keyword paths: if None refresh all, else list of specific directories

        """
        with self._refreshCond:
            if paths is not None:
                self._refreshDirs = list()
                for dobj in paths:
                    self._refreshDirs.append(dobj)
            self._refreshCond.notify()

    def Shutdown(self):
        """Shut the thread down"""
        self._continue = False

    def Suspend(self):
        """Suspend the thread"""
        self._suspend = True

    def Continue(self):
        """Continue the thread"""
        self._suspend = False
        with self._suspendcond:
            self._suspendcond.notify()



###############################################################################
# Name: testSynXml.py                                                         #
# Purpose: Unit tests for the Syntax Xml Library                              #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""Unittest cases for the Syntax Xml Library"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: testSynXml.py 68817 2011-08-21 20:16:52Z CJP $"
__revision__ = "$Revision: 68817 $"

#-----------------------------------------------------------------------------#
# Imports
import wx.stc
import unittest

# Local modules
import common

# Module to test
import syntax

#-----------------------------------------------------------------------------#
# Test Class

class SynXmlTest(unittest.TestCase):
    def setUp(self):
        self.path = common.GetDataFilePath(u'syntax.xml')
        self.bad = common.GetDataFilePath(u'bad.xml')
        self.xml = common.GetFileContents(self.path)
        self.badxml = common.GetFileContents(self.bad)

        self.fhandle = syntax.SyntaxModeHandler(self.path)
        self.fhandle.LoadFromDisk()

    def tearDown(self):
        pass

    #---- Base Tests ----#

    def testGetXml(self):
        """Test getting the xml representation"""
        tmp_h = syntax.SyntaxModeHandler()
        tmp_h.LoadFromString(self.xml)

        x1 = self.fhandle.GetXml()
        x2 = tmp_h.GetXml()
        self.assertEquals(x1, x2)

    #---- SyntaxModeHandler tests ----#

    def testGetCommentPattern(self):
        """Get retrieving the comment pattern from the xml"""
        self.assertEquals(self.fhandle.CommentPattern,
                          self.fhandle.GetCommentPattern())
        self.assertEquals(self.fhandle.CommentPattern, [u"#",],
                          "pattern is: %s" % self.fhandle.CommentPattern)

    def testGetFeatureFromXml(self):
        """Test retrieving a feature"""
        feat = self.fhandle.GetFeatureFromXml(u'AutoIndenter')
        self.assertEquals(feat, u'myextension.py')

    def testGetFileExtensions(self):
        """Test getting the file extension list"""
        exts = self.fhandle.GetFileExtensions()
        self.assertTrue(isinstance(exts, list))
        for ext in exts:
            self.assertTrue(isinstance(ext, basestring))

    def testGetKeywords(self):
        """Test getting the keywords from the xml"""
        kwords = self.fhandle.GetKeywords()
        self.assertTrue(isinstance(kwords, list), "Not a List")
        self.assertTrue(isinstance(kwords[0], tuple), "Not a tuple")
        self.assertTrue(len(kwords) == 2)
        self.assertEquals(kwords[0][0], 0, "not sorted")
        self.assertTrue(u'else' in kwords[0][1])
        self.assertTrue(u'str' in kwords[1][1])

    def testGetLangId(self):
        """Test getting the language identifier string"""
        lid = self.fhandle.GetLangId()
        self.assertEquals(lid, 'ID_LANG_PYTHON')

    def testGetProperties(self):
        """Test getting the property list from the xml"""
        props = self.fhandle.GetProperties()
        self.assertTrue(isinstance(props, list))
        self.assertTrue(isinstance(props[0], tuple))
        self.assertTrue(len(props) == 2)
        self.assertEquals(props[0][0], "fold")
        self.assertEquals(props[0][1], "1")

    def testGetSyntaxSpec(self):
        """Test getting the syntax specifications from the xml"""
        spec = self.fhandle.GetSyntaxSpec()
        self.assertTrue(isinstance(spec, list))
        self.assertTrue(isinstance(spec[0], tuple))
        self.assertTrue(len(spec) == 2)
        self.assertEquals(spec[0][0], wx.stc.STC_P_DEFAULT,
                          "val was: %s" % spec[0][0])
        self.assertEquals(spec[0][1], "default_style",
                          "val was: %s" % spec[0][1])
        self.assertEquals(spec[1][0], wx.stc.STC_P_WORD,
                          "val was: %s" % spec[0][0])
        self.assertEquals(spec[1][1], "keyword_style",
                          "val was: %s" % spec[0][1])

    def testIsOk(self):
        """Test IsOk method"""
        tmp_h = syntax.SyntaxModeHandler(self.path)
        tmp_h.LoadFromDisk()
        self.assertTrue(tmp_h.IsOk())
        self.assertTrue(tmp_h.Ok)
        tmp_h = syntax.SyntaxModeHandler(self.bad)
        tmp_h.LoadFromDisk()
        self.assertFalse(tmp_h.IsOk())
        self.assertFalse(tmp_h.Ok)

    def testVersion(self):
        """Test the xml version attribute"""
        self.assertEquals(self.fhandle.Version, 1,
                        "val was: " + str(self.fhandle.Version))
        



###############################################################################
# Name: ed_event.py                                                           #
# Purpose: Custom events used by Editra                                       #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
Provides custom events for the editors controls/objects to utilize

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: ed_event.py 63789 2010-03-30 02:25:17Z CJP $"
__revision__ = "$Revision: 63789 $"

#-----------------------------------------------------------------------------#
# Dependencies
import wx

#-----------------------------------------------------------------------------#

edEVT_UPDATE_TEXT = wx.NewEventType()
EVT_UPDATE_TEXT = wx.PyEventBinder(edEVT_UPDATE_TEXT, 1)
class UpdateTextEvent(wx.PyCommandEvent):
    """Event to signal that text needs updating"""
    def __init__(self, etype, eid, value=None):
        """Creates the event object"""
        wx.PyCommandEvent.__init__(self, etype, eid)
        self._value = value

    def GetValue(self):
        """Returns the value from the event.
        @return: the value of this event

        """
        return self._value

#--------------------------------------------------------------------------#

edEVT_NOTIFY = wx.NewEventType()
EVT_NOTIFY = wx.PyEventBinder(edEVT_NOTIFY, 1)
class NotificationEvent(UpdateTextEvent):
    """General notification event"""
    def __init__(self, etype, eid, value=None, obj=None):
        UpdateTextEvent.__init__(self, etype, eid, value)
        self.SetEventObject(obj)

#--------------------------------------------------------------------------#

edEVT_MAINWINDOW_EXIT = wx.NewEventType()
EVT_MAINWINDOW_EXIT = wx.PyEventBinder(edEVT_MAINWINDOW_EXIT, 1)
class MainWindowExitEvent(wx.PyCommandEvent):
    """Event to signal that the main window is exiting"""
    pass

#--------------------------------------------------------------------------#

edEVT_STATUS = wx.NewEventType()
EVT_STATUS = wx.PyEventBinder(edEVT_STATUS, 1)
class StatusEvent(wx.PyCommandEvent):
    """Event for posting status events"""
    def __init__(self, etype, eid, msg=None, sec=0):
        """Create an event that can be used to post status messages
        to the main windows status bar.
        @param etype: The type of event to create
        @param eid: The event id
        @keyword msg: The status message to post with the event
        @keyword sec: The section of the status bar to post message to

        """
        wx.PyCommandEvent.__init__(self, etype, eid)
        self._msg = msg
        self._sec = sec

    def GetMessage(self):
        """Returns the value from the event.
        @return: the value of this event

        """
        return self._msg

    def GetSection(self):
        """Returns the messages posting section
        @return: int zero based index of where to post to statusbar

        """
        return self._sec



###############################################################################
# Name: iface.py                                                              #
# Purpose: Plugin interface definitions                                       #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
This module contains numerous plugin interfaces and the Extension points that
they extend. Included below is a list of interfaces available in this module.

Intefaces:
  - ShelfI: Interface into the L{Shelf}
  - MainWindowI: Interface into L{ed_main.MainWindow}
  - AutoCompI: Interface for adding autocompletion helpers

@summary: Main Plugin interface defintions

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: iface.py 70229 2012-01-01 01:27:10Z CJP $"
__revision__ = "$Revision: 70229 $"

#--------------------------------------------------------------------------#
# Imports
import wx

# Local Imports
import plugin

#--------------------------------------------------------------------------#

class AutoCompI(plugin.Interface):
    """The Autocompletion interface.

    """
    def GetCompleter(self, buff):
        """Get the completer object implemented by this plugin
        @param buff: EditraStc instance
        @return: instance of autocomp.BaseCompleter

        """
        raise NotImplementedError

    def GetFileTypeId(self):
        """Get the filetype this completer is associated with
        @return: int

        """
        return 0

#--------------------------------------------------------------------------#

class MainWindowI(plugin.Interface):
    """The MainWindow Interface is intended as a simple general purpose
    interface for adding functionality to the main window. It does little
    managing of how objects that implement it are handled, most is left up to
    the plugin. Some examples of plugins using this interface are the
    FileBrowser and Calculator plugins.

    """
    def PlugIt(self, window):
        """This method is called once and only once per window when it is 
        created. It should typically be used to register menu entries, 
        bind event handlers and other similar actions.
        @param window: The parent window of the plugin
        @postcondition: The plugins controls are installed in the L{MainWindow}

        """
        raise NotImplementedError

    def GetMenuHandlers(self):
        """Get menu event handlers/id pairs. This function should return a
        list of tuples containing menu ids and their handlers. The handlers
        should be not be a member of this class but a member of the ui component
        that they handler acts upon.
        @return: list [(ID_FOO, foo.OnFoo), (ID_BAR, bar.OnBar)]

        """
        pass

    def GetUIHandlers(self):
        """Get update ui event handlers/id pairs. This function should return a
        list of tuples containing object ids and their handlers. The handlers
        should be not be a member of this class but a member of the ui component
        that they handler acts upon.
        @return: list [(ID_FOO, foo.OnFoo), (ID_BAR, bar.OnBar)]

        """
        pass

#-----------------------------------------------------------------------------#

class ShelfI(plugin.Interface):
    """Interface into the L{Shelf}. All plugins wanting to be
    placed on the L{Shelf} should implement this interface.

    """
    def AllowMultiple(self):
        """This method is used to check if multiple instances of this
        item are allowed to be open at one time.
        @return: True/False

        """
        return True

    def CreateItem(self, parent):
        """This is them method used to open the item in the L{Shelf}
        It should return an object that is a Panel or subclass of a Panel.
        @param parent: The would be parent window of this panel
        @return: wx.Panel

        """
        raise NotImplementedError

    def GetBitmap(self):
        """Get the bitmap to show in the shelf for this item
        @return: wx.Bitmap
        @note: this method is optional

        """
        return wx.NullBitmap

    def GetId(self):
        """Return the id that identifies this item (same as the menuid)
        @return: Item ID

        """
        raise NotImplementedError

    def GetMenuEntry(self, menu):
        """Returns the menu entry associated with this item
        @param menu: The menu this entry will be added to
        @return: wx.MenuItem or None if no menu entry is needed

        """
        raise NotImplementedError

    def GetName(self):
        """Return the name of this shelf item. This should be the
        same as the MenuEntry's label.
        @return: name of item

        """
        raise NotImplementedError

    def InstallComponents(self, mainw):
        """Called by the Shelf when the plugin is created to allow it
        to install any extra components that it may have that fall outside
        the normal interface. This method is optional and does not need
        to be implemented if it is not needed.
        @param mainw: MainWindow Instance

        """
        pass

    def IsStockable(self):
        """Return whether this item type is stockable. The shelf saves
        what pages it had open the last time the program was run and then
        reloads the pages the next time the program starts. If this
        item can be reloaded between sessions return True otherwise return
        False.
        @return: bool

        """
        return True

#-----------------------------------------------------------------------------#



################################################################################
## Name: runUnitTests.py                                                       #
## Purpose: Run the Unit Test Suite                                            #
## Author: Cody Precord <cprecord@editra.org>                                  #
## Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
## License: wxWindows License                                                  #
################################################################################

"""
Run Editra's Unittest Suite.

This module is mostly copied from the wxPython Unittest Suite.

@summary: Unittest Suite Main Module

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: runUnitTests.py 65165 2010-08-02 22:04:37Z CJP $"
__revision__ = "$Revision: 65165 $"

#-----------------------------------------------------------------------------#
# Imports
import os
import sys
import unittest
import time
import wx
from optparse import OptionParser

# Put Editra/src on the path
sys.path.append(os.path.abspath("../../src"))
sys.path.append(os.path.abspath("../../src/extern"))

# Local Utilities
import common

# ----------------- Helper Functions / Classes ---------------------

# TODO: maybe change some variable names?
# TODO: maybe put this function as a method somewhere else?
def _make_clean_opt_string():
    # which options was this run called with?
    # replace short opts with long opts (explicit is better than implicit)
    opt_string = ""
    args = sys.argv[1:]
    for arg in args:
        if arg.startswith("-") and not arg.startswith("--"):
            # handle the case where opt and arg are conjoined
            arg2 = None
            if len(arg) > 2:
                arg2 = arg[2:]
                arg  = arg[:2]
            # it's a short opt, now find it
            for opt in parser.option_list:
                if arg in opt._short_opts:
                    opt_string += opt._long_opts[0]
                    if opt.action == "store":
                        opt_string += "="
                        if arg2 != None:
                            opt_string += arg2
                    else:
                        opt_string += " "
        else:
            opt_string += arg
            opt_string += " "
    if opt_string == "":
        opt_string = "NONE"
    return opt_string

def output(string):
    print string

#-----------------------------------------------------------------------------#

class UnitTestSuite:
    def __init__(self, include="", exclude="", tests=""):
        # error checking
        if include != "" and exclude != "":
            raise ValueError("include and exclude arguments are mutually exclusive")
        # TODO: could this become a simple os.listdir(".")?
        _rootdir = os.path.abspath(sys.path[0])
        if not os.path.isdir(_rootdir):
            _rootdir = os.path.dirname(_rootdir)
        self.rootdir = _rootdir # to come in handy later
        # a dict of all possible test modules that could be run
        # ASSUME: each module name is unique not solely because of case
        _module_names = {}
        for _name in [ n[:-3] for n in os.listdir(self.rootdir)
                    if n.startswith("test") and n.endswith(".py") ]:
            _module_names[ _name.lower() ] = _name
        # make the include/exclude/tests lists
        _module_specs = None
        _spec_type    = None
        _test_specs   = None
        if include != "":
            _module_specs = self._clean_listify(include)
            _spec_type    = "include"
        elif exclude != "":
            _module_specs = self._clean_listify(exclude)
            _spec_type    = "exclude"

        if tests != "":
            _test_specs = self._clean_listify(tests, False)

        # make sure they all exist
        if _module_specs != None: # TODO: got to be a better place to put this
            for _mod in _module_specs:
                if not _module_names.has_key(_mod.lower()):
                    parser.error("Module %s not found under test" % (_mod))

        # now import the modules
        if _module_specs == None:
            self.modules = [ __import__(name) for name in _module_names.values() ]
        elif _spec_type == "include":
            self.modules = [ __import__(name) for name in _module_specs ]
        elif _spec_type == "exclude":
            self.modules = [ __import__(name) for name in _module_names.values()
                                            if name not in _module_specs ]
        # convert modules into suites
        self.suites = []
        for module in self.modules:
            _classname = module.__name__[4:] + "Test"
            _class = module.__getattribute__(_classname)
            # build test suite (whether or not --tests are specified)
            if _test_specs == None:
                _suite = unittest.makeSuite(_class)
            else:
                _suite = unittest.TestSuite()
                for _test_name in unittest.getTestCaseNames(_class,"test"):
                    for _test in _test_specs:
                        _docstr = getattr(_class, _test_name).__doc__
                        if _test_name.lower().find(_test.lower()) != -1 or \
                               _docstr != None and _docstr.lower().find(_test.lower()) != -1:
                            _suite.addTest(_class(_test_name))
                        break

            # filter out tests that shouldn't be run in subclasses
            _tests = _suite._tests
            for _t in _tests:
                # TODO: pull logic into wxtest
                # or use the version of unittest instead
                if sys.version_info[0:2] >= (2,5):
                    _mname = _t._testMethodName
                else:
                    _mname = _t._TestCase__testMethodName
                
                if _mname.find('_wx') != -1:
                    # grab the class: everything between '_wx' and 'Only' at the end
                    restriction = _mname[_mname.find('_wx')+3:-4]
                    if not _class.__name__.startswith(restriction):
                        #print "filtered: %s (class=%s)" % (mname,_class.__name__)
                        _tests.remove(_t)

            # if suite is non-empty...
            if _suite.countTestCases() > 0:
                # add it to the list of suites :-)
                self.suites.append(_suite)
    
    def _clean_listify(self, string, include_or_exclude=True):
        _clean_list = []
        _list = string.split(",")
        for s in _list:
            if include_or_exclude:
                if s.endswith(".py"):
                    s = s[:-3]
                if s.startswith("wx."):
                    s = "test" + s[3:]
                if not s.startswith("test"):
                    s = "test" + s
            _clean_list.append(s)
        # maintains capitalization
        return _clean_list
    
    def _start_figleaf(self):
        if options.figleaf != "":
            globals()["figleaf"] = __import__("figleaf")
            # TODO: perhaps make this class-specific rather than global?
            globals()["figfile"] = os.path.join(self.rootdir, options.figleaf_filename)
            if os.path.exists(figfile):
                os.remove(figfile)
            figleaf.start(ignore_python_lib=False)

    def _stop_figleaf(self):
        if options.figleaf != "":
            figleaf.stop()
            figleaf.write_coverage(figfile)
        
    def run(self):
        test_run_data = UnitTestRunData()
        self._start_figleaf()
        self.start_time = time.time()
        # run tests
        for _suite in self.suites:
            _result = unittest.TestResult()
            _suite.run(_result)
            _module_name = _suite._tests[0].__module__
            test_run_data.addResult(_module_name, _result)
        self.stop_time = time.time()
        self._stop_figleaf()
        # process results
        test_run_data.setTime(self.start_time, self.stop_time)
        test_run_data.process()
        # return results
        return test_run_data

#-----------------------------------------------------------------------------#

class UnitTestRunData:
    def __init__(self):
        self.results = {}
    
    def addResult(self, module_name, result):
        self.results[module_name] = result
    
    def setTime(self, start, stop):
        self.startTime = start
        self.stopTime  = stop
    
    def process(self):
        # process data
        self.elapsedTime = self.stopTime - self.startTime
        self.countSuites = len(self.results)
        self.countSuccesses = 0
        self.countFailures  = 0
        self.countErrors    = 0
        self.rawData = {}
        for _module_name, _result in self.results.iteritems():
            # TODO: revisit all this processing, is everything necessary?
            tmp = {}
            # parse results individually
            tmp["failures"]  = len(_result.failures)
            tmp["errors"]    = len(_result.errors)
            tmp["successes"] = _result.testsRun - tmp["failures"] - tmp["errors"]
            # total results
            self.countSuccesses += tmp["successes"]
            self.countFailures  += tmp["failures"]
            self.countErrors    += tmp["errors"]
            # TODO: add processing here
            tmp["failure_data"] = _result.failures
            tmp["error_data"]   = _result.errors
            self.rawData[_module_name] = tmp
        
# -----------------------------------------------------------
# -------------------- Option Logic -------------------------

# Options
usage = "usage: python %prog [options]"
parser = OptionParser(usage=usage)
parser.add_option("-o", "--output-filename", default="",
                    action="store", dest="outfilename",
                    metavar="FILE",
                    help="redirect output from console to FILE")
parser.add_option("-f", "--figleaf", default="",
                    action="store", dest="figleaf", metavar="FILE",
                    help="use the figleaf code-coverage tool, and write figleaf output to " +
                    "FILE. you must have figleaf installed to use this option. " +
                    "using this option will result in a slower test run")
parser.add_option("-i", "--include-modules", default="",
                    action="store", dest="module_list",
                    help="run only the comma-separated list of modules given. use either " +
                         "wx class names or the name of the desired test module. " + 
                         "don't use spaces in the list")
parser.add_option("-e", "--exclude-modules", default="",
                    action="store", dest="module_ex_list",
                    help="run all modules excluding those given in the comma-separated " + 
                    "list given. use either wx class names or the name of the desired " +
                    "test module.")
parser.add_option("-t", "--tests", default="",
                    action="store", dest="test_list",
                    help="run only a targeted list of tests. give a comma-separated list " +
                            "of strings, and each test whose name or docstring contains " +
                            "one of those given will be run.")

def runUnitTestsAndOutputResults():   
    unit_test_suite = UnitTestSuite(include=options.module_list,
                                    exclude=options.module_ex_list,
                                    tests=options.test_list)
    
    result_data = unit_test_suite.run()
    
    # see refactored method above
    opt_string = _make_clean_opt_string()
    
    # -----------------------------------------------------------
    # ------------------- Output Reporting ----------------------
    output("") # make things easier to read
    output("%s - %s\n" % (time.asctime(), wx.GetOsDescription()))
    output("Platform Information")
    output("Platform [sys.platform]: %s" % sys.platform)
    output("Python Version [sys.version]: %s" % sys.version)
    output("wx Version [wx.version()]: %s" % wx.version())
    output("OS [wx.GetOsDescription()]: %s" % wx.GetOsDescription())
    output("wx Info [wx.PlatformInfo]: %s" % str(wx.PlatformInfo))
    output("runUnitTests.py options: %s" % opt_string)
    output("\n----------------------\n")
    
    output("Summary")
    output("Run completed in %.2f seconds" % (result_data.elapsedTime))
    output("%d classes tested" % (result_data.countSuites))
    output("%d tests passed in total!" % (result_data.countSuccesses))
    if result_data.countFailures > 0:
        output("%d tests failed in total!" % (result_data.countFailures))
    if result_data.countErrors > 0:
        output("%d tests erred in total!" % (result_data.countErrors))
    output("\n----------------------\n")
    
    data_items = result_data.rawData.items()
    data_items.sort()
    
    output("Module Data")
    for mod_name, results in data_items:
        messages = ["%d passed" % (results["successes"])]
        if results["failures"] > 0:
            messages.append("%d failed" % (results["failures"]))
        if results["errors"] > 0:
            messages.append("%d erred"  % (results["errors"]))
        output("%s:  %s" % (mod_name, ", ".join(messages)))
    output("\n----------------------\n")
    
    if result_data.countFailures + result_data.countErrors > 0:
        output("Failure Data")
    for mod_name, results in data_items:
        # report on it
        for failure in results["failure_data"] + results["error_data"]:
            type = None
            if failure in results["failure_data"]:
                type = "Fail: "
            elif failure in results["error_data"]:
                type = "Error: "
            output("   " + type + str(failure[0]))
            output("      " + str(failure[1]).replace("\n","\n      "))

#-----------------------------------------------------------------------------#
if __name__ == '__main__':
    (options, args) = parser.parse_args()

    # Options error-checking
    if options.module_list != "" and options.module_ex_list != "":
        parser.error("options --exclude-modules and --include-modules are mutually exclusive")
        
    # File redirect
    if options.outfilename != "":
        origstdout = sys.stdout
        try:
            sys.stdout = open(options.outfilename,'w')
        except IOError:
            print "Error opening output file, defaulting to original stdout"
            sys.stdout = origstdout

    app = common.EdApp(False)
    runUnitTestsAndOutputResults()
#    app.MainLoop()




#!/usr/bin/env python
###############################################################################
# Name: Editra.pyw                                                            #
# Purpose: Editra's main launch script                                        #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################


"""
Main launch script for the Editor. It first tries to look for Editra on the
local path and if it is not there it tries to import the Main method
from where Editra would be installed if it was installed using distutils

@summary: Editra's main launch script for Windows

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: Editra.pyw 63538 2010-02-23 03:39:03Z CJP $"
__revision__ = "$Revision: 63538 $"

#--------------------------------------------------------------------------#
# Dependencies
import sys
import os

try:
    import src as esrc
    IS_LOCAL = True
except ImportError:
    try:
        import Editra as esrc
        IS_LOCAL = False
    except ImportError, msg:
        print "There was an error while tring to import Editra"
        print ("Make sure that Editra is on your PYTHONPATH and that "
               "you have wxPython installed.")
        print "ERROR MSG: "
        print str(msg)
        os._exit(1)

#--------------------------------------------------------------------------#
# There are currently some necessary hacks for launching editra from this
# script that will hopefully be removed in the not so distance future once
# the plugin managers meta registry is redesigned.

def main():
    # The initial import above is necessary to get the path of where
    # Editra is installed so that the src package can be put on the path.
    # If the src module is not on the path the plugins are unable to import
    # things from inside editras namespace properly. It also causes problems
    # with recongnizing plugins in Extension registry of the Plugins metaclass.
    SRC_DIR = os.path.dirname(esrc.__file__)
    if not IS_LOCAL:
        SRC_DIR = os.path.join(SRC_DIR, 'src')

    # Cleanup any of Editras modules that are already present before
    # importing Editra again so that the modules are imported with the
    # correct signature (i.e ed_theme vs src.ed_theme). As the plugin
    # manager currently registers the class objects metadata by using
    # the classes module signature for identification.
    if not IS_LOCAL:
        torem = [ key for key in sys.modules.keys()
                  if key.startswith('Editra') ]
        for key in torem:
            del sys.modules[key]
    else:
        if 'src' in sys.modules:
            del sys.modules['src']

    sys.path.insert(0, SRC_DIR)
    import Editra
    Editra.Main()


if __name__ == '__main__':
    main()



###############################################################################
# Name: AdvFindReplaceDialogDemo.py                                           #
# Purpose: Advanced Find Replace Dialog Test and Demo File                    #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""
Test file for testing the Advanced Find Replace Dialog

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: AdvFindReplaceDialogDemo.py 62424 2009-10-16 02:05:13Z CJP $"
__revision__ = "$Revision: 62424 $"

#-----------------------------------------------------------------------------#
# Imports
import sys
import os
import wx

#sys.path.insert(0, os.path.abspath('../../src'))
import eclib

#-----------------------------------------------------------------------------#
ID_DEFAULT = wx.NewId()
ID_DIALOG  = wx.NewId()
ID_REPLACE = wx.NewId()
ID_NOUPDOWN = wx.NewId()
ID_NOOPTS  = wx.NewId()
ID_REGEX   = wx.NewId()
ID_NOLOOK  = wx.NewId()
ID_NOWHOLEW = wx.NewId()
ID_NO_COUNT = wx.NewId()
ID_NO_ALL_ACT = wx.NewId()
ID_NO_MODE_SELECT = wx.NewId()
ID_MINIMAL = wx.NewId()

#-----------------------------------------------------------------------------#
# There are a large number of possible flag/style combinations here are just a
# few examples for creating different versions of the dialog.
DIALOG_MAP = {ID_DEFAULT : (wx.FindReplaceData(),
                            eclib.AFR_STYLE_FINDDIALOG,
                            "Default Find Dialog"),
              ID_DIALOG  : (wx.FindReplaceData(),
                            eclib.AFR_STYLE_NON_FLOATING,
                            "Non Floating Find Dialog"),
              ID_REPLACE : (wx.FindReplaceData(),
                            eclib.AFR_STYLE_REPLACEDIALOG,
                            "Default Replace Dialog"),
              ID_NOUPDOWN : (wx.FindReplaceData(eclib.AFR_NOUPDOWN),
                            eclib.AFR_STYLE_REPLACEDIALOG,
                            "No Up/Down Option"),
              ID_NOOPTS  : (wx.FindReplaceData(eclib.AFR_NOOPTIONS),
                           eclib.AFR_STYLE_FINDDIALOG,
                           "Options Hidden"),
              ID_REGEX   : (wx.FindReplaceData(eclib.AFR_REGEX),
                            eclib.AFR_STYLE_FINDDIALOG,
                            "Regex Selected"),
              ID_NOLOOK  : (wx.FindReplaceData(eclib.AFR_NOLOOKIN),
                            eclib.AFR_STYLE_FINDDIALOG,
                            "Lookin Hidden"),
              ID_NOWHOLEW : (wx.FindReplaceData(eclib.AFR_NOWHOLEWORD),
                             eclib.AFR_STYLE_FINDDIALOG,
                             "Whole Word Disabled"),
              ID_NO_COUNT : (wx.FindReplaceData(eclib.AFR_NO_COUNT),
                             eclib.AFR_STYLE_FINDDIALOG,
                             "No Count Button"),
              ID_NO_ALL_ACT : (wx.FindReplaceData(eclib.AFR_NO_ALL_BTN),
                             eclib.AFR_STYLE_FINDDIALOG,
                             "No Find/Replace All"),
              ID_NO_MODE_SELECT : (wx.FindReplaceData(),
                             eclib.AFR_STYLE_FINDDIALOG|eclib.AFR_STYLE_NO_MODE_SELECT,
                             "No Mode Select"),
              ID_MINIMAL : (wx.FindReplaceData(eclib.AFR_SIMPLE),
                            eclib.AFR_STYLE_FINDDIALOG,
                            "All Options Hidden")}

BUTTONS = [(ID_DEFAULT, "Default"), (ID_DIALOG, "Non-Floating"),
           (ID_REPLACE, "Replace Dialog"), (ID_NOUPDOWN, "No Up/Down Option"),
           (ID_NOOPTS, "Options Hidden"),
           (ID_REGEX, "Regular Expression"), (ID_NOLOOK, "Lookin Hidden"),
           (ID_NOWHOLEW, "Whole Word Disabled"), (ID_NO_COUNT, "No Count Button"),
           (ID_NO_ALL_ACT, "No Find/Replace All"),
           (ID_NO_MODE_SELECT, "No Mode Select"), (ID_MINIMAL, "Minimal")]

#-----------------------------------------------------------------------------#

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        wx.Panel.__init__(self, parent)

        # Attributes
        self.data = None
        self.dlg = None
        self.log = log

        # Layout
        self.__DoLayout()

        # Event Handlers
        self.Bind(wx.EVT_BUTTON, self.OnButton)
        self.Bind(eclib.EVT_FIND, self.OnFind)
        self.Bind(eclib.EVT_FIND_ALL, self.OnFind)
        self.Bind(eclib.EVT_FIND_NEXT, self.OnFind)
        self.Bind(eclib.EVT_REPLACE, self.OnFind)
        self.Bind(eclib.EVT_REPLACE_ALL, self.OnFind)
        self.Bind(eclib.EVT_FIND_CLOSE, self.OnFindClose)

    def __DoLayout(self):
        """Layout the panel"""
        fsizer = wx.BoxSizer(wx.VERTICAL)
        fsizer.Add((10, 10), 0)
        for bid, lbl in BUTTONS:
            btn = wx.Button(self, bid, lbl)
            fsizer.Add(btn, 0)
            fsizer.Add((5, 5), 0)
        msizer = wx.BoxSizer(wx.HORIZONTAL)
        msizer.AddMany([((10, 10), 0), (fsizer, 0, wx.EXPAND), ((10, 10), 0)])
        self.SetSizer(msizer)
        self.SetAutoLayout(True)

    def OnButton(self, evt):
        """Show a dialog"""
        e_id = evt.GetId()
        if e_id in DIALOG_MAP:
            self.data, style, title = DIALOG_MAP[e_id]
            self.dlg = eclib.AdvFindReplaceDlg(self, self.data, title, style)
            self.dlg.CenterOnParent()
            self.dlg.Show()
        else:
            evt.Skip()

    def OnFind(self, evt):
        self.log.write("Search String: %s" % evt.GetFindString())
        self.log.write("Replace String: %s" % evt.GetReplaceString())
        self.log.write("Location: %s" % evt.GetDirectory())
        self.log.write("Search Type: %d" % evt.GetSearchType())
        self.log.write("Whole Word: %d" % evt.IsWholeWord())
        self.log.write("Match Case: %d" % evt.IsMatchCase())
        self.log.write("Regular Expression: %d" % evt.IsRegEx())
        self.log.write("Search Up: %d" % evt.IsUp())
        self.log.write("EvtType: %d" % evt.GetEventType())

    def OnFindClose(self, evt):
        """Called when dialog closes"""
        if self.dlg is not None:
            self.log.write("Dialog Closed")
            self.dlg.Destroy()
            self.dlg = None
        
#-----------------------------------------------------------------------------#

class TestLog:
    def __init__(self):
        pass

    def write(self, msg):
        print msg

#-----------------------------------------------------------------------------#

overview = eclib.finddlg.__doc__
title = "AdvFindReplaceDlg"

#-----------------------------------------------------------------------------#
if __name__ == '__main__':
    try:
        import run
    except ImportError:
        app = wx.PySimpleApp(False)
        frame = wx.Frame(None, title="Advanced Find Dialog Test")
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(TestPanel(frame, TestLog()), 1, wx.EXPAND)
        frame.CreateStatusBar()
        frame.SetSizer(sizer)
        frame.SetInitialSize(wx.Size(350, 350))
        frame.CenterOnParent()
        frame.Show()
        app.MainLoop()
    else:
        run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])



###############################################################################
# Name: testFactory.py                                                        #
# Purpose: Unit tests for the FactoryMixin.                                   #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2011 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""Unittest cases for testing ebmlib.FactoryMixin"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: testFactory.py 69448 2011-10-18 14:17:53Z CJP $"
__revision__ = "$Revision: 69448 $"

#-----------------------------------------------------------------------------#
# Imports
import unittest

# Module to test
import ebmlib

#-----------------------------------------------------------------------------#
# Test Class

class FactoryTest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testFactoryCreate(self):
        obj = BaseWidget.FactoryCreate('foo')
        self.assertTrue(isinstance(obj, FooWidget))
        self.assertTrue(obj.meta.data == 'FOOBAR')
        obj = BaseWidget.FactoryCreate('bar')
        self.assertTrue(isinstance(obj, BarWidget))
        self.assertTrue(obj.meta.data == 'NULL')
        obj = BaseWidget.FactoryCreate('junk')
        self.assertTrue(type(obj) is BaseWidget)

#-----------------------------------------------------------------------------#

# Test classes to test factory

class BaseWidget(ebmlib.FactoryMixin):
    def __init__(self):
        super(BaseWidget, self).__init__()

    @classmethod
    def GetMetaDefaults(cls):
        return dict(id=None, data="NULL")

class FooWidget(BaseWidget):
    class meta:
        id = "foo"
        data = "FOOBAR"

class BarWidget(BaseWidget):
    class meta:
        id = "bar"



 #-*- coding: utf-8 -*-
###############################################################################
# Name: syndata.py                                                            #
# Purpose: Syntax Data Base                                                   #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

""" Interface definition for syntax data

@summary: Editra Syntax Data Interface Definition

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: syndata.py 67392 2011-04-04 21:06:43Z CJP $"
__revision__ = "$Revision: 67392 $"

__all__ = ['SyntaxDataBase',]

#-----------------------------------------------------------------------------#
# Imports
import collections
import wx.stc as stc

# Local Imports
import synglob

#-----------------------------------------------------------------------------#

class SyntaxDataBase(object):
    """Syntax data container object base class"""
    def __init__(self, langid=synglob.ID_LANG_TXT):
        object.__init__(self)

        # Attributes
        self._langid = langid
        self._lexer = stc.STC_LEX_NULL
        self._features = dict()

    @property
    def CommentPattern(self):
        return self.GetCommentPattern()

    @property
    def Keywords(self):
        return self.GetKeywords()

    @property
    def LangId(self):
        return self.GetLangId()

    @property
    def Lexer(self):
        return self.GetLexer()

    @property
    def Properties(self):
        return self.GetProperties()

    @property
    def SyntaxSpec(self):
        return self.GetSyntaxSpec()

    #---- Interface Methods ----#

    def GetCommentPattern(self):
        """Get the comment pattern
        @return: list of strings ['/*', '*/']

        """
        return list()

    def GetKeywords(self):
        """Get the Keyword List(s)
        @return: list of tuples [(1, ['kw1', kw2']),]

        """
        return list()

    def GetLangId(self):
        """Get the language id
        @return: int

        """
        return self._langid

    def GetLexer(self):
        """Get the lexer id
        @return: wx.stc.STC_LEX_

        """
        return self._lexer

    def GetProperties(self):
        """Get the Properties List
        @return: list of tuples [('fold', '1'),]

        """
        return list()

    def GetSyntaxSpec(self):
        """Get the the syntax specification list
        @return: list of tuples [(int, 'style_tag'),]
        @note: required override for subclasses

        """
        raise NotImplementedError

    #---- End Interface Methods ----#

    def GetFeature(self, name):
        """Get a registered features callable
        @param name: feature name
        @return: callable or None

        """
        return self._features.get(name, None)

    def RegisterFeature(self, name, funct):
        """Register an extension feature with the factory
        @param name: feature name
        @param funct: callable

        """
        assert isinstance(funct, collections.Callable), "funct must be callable object"
        self._features[name] = funct

    def SetLexer(self, lex):
        """Set the lexer object for this data object"""
        self._lexer = lex

    def SetLangId(self, lid):
        """Set the language identifier
        @param lid: int

        """
        self._langid = lid

#-----------------------------------------------------------------------------#



###############################################################################
# Name: testSynGlob.py                                                        #
# Purpose: Unit tests for syntax.synglob utilities                            #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""Unittest cases for testing syntax.synglob utilities and functions"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: testSynGlob.py 62572 2009-11-08 19:16:11Z CJP $"
__revision__ = "$Revision: 62572 $"

#-----------------------------------------------------------------------------#
# Imports
import unittest

# Module to test
import syntax.synglob as synglob

#-----------------------------------------------------------------------------#
# Test Class

class SynGlobTest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    #---- Test Cases ----#

    def testGetDescriptionFromId(self):
        """Test GetDescriptionFromId"""
        desc = synglob.GetDescriptionFromId(synglob.ID_LANG_PYTHON)
        self.assertEquals(desc, synglob.LANG_PYTHON)

        desc = synglob.GetDescriptionFromId(synglob.ID_LANG_JAVA)
        self.assertEquals(desc, synglob.LANG_JAVA)

        # Test that some unknown id's always return Plain Text
        desc = synglob.GetDescriptionFromId(0)
        self.assertEquals(desc, synglob.LANG_TXT)

        desc = synglob.GetDescriptionFromId(100)
        self.assertEquals(desc, synglob.LANG_TXT)

    def testGetIdFromDescription(self):
        """Get getting a language id from its description string"""
        id_ = synglob.GetIdFromDescription(u"Python")
        self.assertEquals(id_, synglob.ID_LANG_PYTHON)

        id_ = synglob.GetIdFromDescription(u"python")
        self.assertEquals(id_, synglob.ID_LANG_PYTHON)

        id_ = synglob.GetIdFromDescription(u"C")
        self.assertEquals(id_, synglob.ID_LANG_C)

        id_ = synglob.GetIdFromDescription(u"SomeFakeLang")
        self.assertEquals(id_, synglob.ID_LANG_TXT)




###############################################################################
# Name: Cody Precord                                                          #
# Purpose: Log File                                                           #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2010 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
Editra Business Model Library: LogFile

Log file class for managing log files or other transient files that should
be purged after a given period of time.

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: logfile.py 66868 2011-02-09 16:01:49Z CJP $"
__revision__ = "$Revision: 66868 $"

__all__ = ['LogFile',]

#--------------------------------------------------------------------------#
# Imports
import os
import time
import datetime
import re
import tempfile

#--------------------------------------------------------------------------#

class LogFile(object):
    """Log file class"""
    def __init__(self, prefix, logdir=None):
        """Create a log file
        @param prefix: filename prefix
        @keyword logdir: abs path to log output dir
        @note: if logdir is None then the system temp directory will be used

        """
        super(LogFile, self).__init__()

        # Attributes
        self.prefix = prefix
        self.logdir = logdir

        # Setup
        if self.logdir is None:
            self.logdir = tempfile.gettempdir()

    #---- Properties ----#
    LogDirectory = property(lambda self: self.logdir,
                            lambda self, dname: setattr(self, 'logdir', dname))
    Prefix = property(lambda self: self.prefix,
                      lambda self, prefix: setattr(self, 'prefix', prefix))

    #---- Public Interface ----#
    def WriteMessage(self, msg):
        """Append the message to the current log file
        @param msg: string object

        """
        # Files are named as prefix_YYYY_MM_DD.log
        logstamp = "%d_%d_%d" % time.localtime()[:3]
        logname = "%s_%s.log" % (self.prefix, logstamp)
        logpath = os.path.join(self.logdir, logname)
        if os.path.exists(logpath):
            opencmd = "ab"
        else:
            opencmd = "wb"

#        with open(logpath, opencmd) as handle:
#            handle.write(msg.rstrip() + os.linesep)
        try:
            handle = open(logpath, opencmd)
            handle.write(msg.rstrip() + os.linesep)
            handle.close()
        except IOError:
            pass

    def PurgeOldLogs(self, days):
        """Purge all log files older than n days
        @param days: number of days

        """
        logpattern = re.compile("%s_[0-9]{4}_[0-9]{1,2}_[0-9]{1,2}.log" % self.prefix)
        paths = list()
        cdate = datetime.date(*time.localtime()[:3])
        for path in os.listdir(self.logdir):
            if logpattern.match(path):
                ymd = [int(x) for x in path[len(self.prefix)+1:-4].split('_')]
                fdate = datetime.date(*ymd)
                span = cdate - fdate
                if span.days > days:
                    fpath = os.path.join(self.logdir, path)
                    paths.append(fpath)

        # Attempt to cleanup the old files
        for log in paths:
            try:
                os.remove(log)
            except OSError:
                pass



###############################################################################
# Name: testFileTypeChecker.py                                                #
# Purpose: Unit tests for ebmlib.FileTypeChecker                              #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""Unittest cases for testing the FileTypeChecker class """

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: testFileTypeChecker.py 65165 2010-08-02 22:04:37Z CJP $"
__revision__ = "$Revision: 65165 $"

#-----------------------------------------------------------------------------#
# Imports
import wx
import os
import unittest

# Local modules
import common

# Module to test
import ebmlib

#-----------------------------------------------------------------------------#
# Test Class

class FileTypeCheckerTest(unittest.TestCase):
    def setUp(self):
        self.fpath = common.GetDataFilePath(u'test_read_utf8.txt')
        self.bpath = common.GetDataFilePath(u'image_test.png')
        self.checker = ebmlib.FileTypeChecker()

    def tearDown(self):
        pass

    #---- Tests ----#
    def testIsBinary(self):
        """Test the IsBinary checker method"""
        self.assertTrue(self.checker.IsBinary(self.bpath))
        self.assertFalse(self.checker.IsBinary(self.fpath))

    def testIsBinaryBytes(self):
        """Test checking for binary bytes in a string"""
        f = open(self.bpath, 'rb')
        bytes = f.read()
        f.close()
        self.assertTrue(self.checker.IsBinaryBytes(bytes))
        f = open(self.fpath, 'rb')
        bytes = f.read()
        f.close()
        self.assertFalse(self.checker.IsBinaryBytes(bytes))

    def testIsReadableText(self):
        """Test if the file is a readable as text."""
        self.assertTrue(self.checker.IsReadableText(self.fpath))




###############################################################################
# Name: testDocMgr.py                                                         #
# Purpose: Unit tests for the doctools module.                                #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""Unittest cases for testing doctools.DocPositionMgr"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: testDocMgr.py 61142 2009-06-20 05:28:36Z CJP $"
__revision__ = "$Revision: 61142 $"

#-----------------------------------------------------------------------------#
# Imports
import wx
import os
import unittest

# Module to test
import doctools

#-----------------------------------------------------------------------------#
# Test Class

class DocMgrTest(unittest.TestCase):
    def setUp(self):
        self.mgr = doctools.DocPositionMgr()

        # Populate some data
        self.mgr.AddRecord(('test.py', 20))
        self.mgr.AddRecord(('test2.py', 100))
        self.mgr.AddRecord(('test3.py', 1200))

    def tearDown(self):
        pass

    #---- Tests ----#
    def testGetBook(self):
        """Test that the path of the book is set"""
        book = self.mgr.GetBook()
        self.assertTrue(book is None, "book == %s" % book)

    def testGetPos(self):
        """Test fetching file positions from the manager."""
        self.assertEqual(self.mgr.GetPos('test2.py'), 100)
        self.assertEqual(self.mgr.GetPos('test3.py'), 1200)
        self.assertNotEqual(self.mgr.GetPos('test.py'), 1200)
        
        # Test trying to get an unknown file
        self.assertEqual(self.mgr.GetPos('fakefile.txt'), 0)

    #-- Test Position Navigator cache --#
    def testCanNavigate(self):
        """Test CanNavigateNext/Prev functions"""
        self.mgr.FlushNaviCache()
        self.assertFalse(self.mgr.CanNavigatePrev())
        self.assertFalse(self.mgr.CanNavigateNext())
        self.mgr.AddNaviPosition('test4.py', 200)
        self.assertTrue(self.mgr.CanNavigatePrev())
        self.mgr.AddNaviPosition('test5.py', 83)
        self.mgr.AddNaviPosition('test5.py', 45)
        self.mgr.AddNaviPosition('test6.py', 998)
        self.assertFalse(self.mgr.CanNavigateNext())
        self.mgr.GetPreviousNaviPos()
        self.mgr.GetPreviousNaviPos()
        self.assertTrue(self.mgr.CanNavigateNext())

    def testGetNextNaviPos(self):
        """Test Getting the next position in the history and retrieving
        items from the navigator cache

        """
        self.mgr.FlushNaviCache()
        self.mgr.AddNaviPosition('test4.py', 200)
        self.mgr.AddNaviPosition('test5.py', 83)
        self.mgr.AddNaviPosition('test5.py', 45)
        self.mgr.AddNaviPosition('test6.py', 998)

        # Move back to begining of cache
        self.mgr.GetPreviousNaviPos()
        self.mgr.GetPreviousNaviPos()
        self.mgr.GetPreviousNaviPos()
        self.mgr.GetPreviousNaviPos()

        pos = self.mgr.GetNextNaviPos() # next in cache
        self.assertEqual(pos[1], 200)

        pos = self.mgr.GetNextNaviPos()
        self.assertEqual(pos[1], 83)

        pos = self.mgr.GetNextNaviPos()
        self.assertEqual(pos[1], 45)

#        pos = self.mgr.GetNextNaviPos('test5.py')
#        self.assertEqual(pos[1], 83)

    def testGetPreviousNaviPos(self):
        """Test Getting the previous position in the history and retrieving
        items from the navigator cache

        """
        self.mgr.FlushNaviCache()
        self.mgr.AddNaviPosition('test4.py', 200)
        self.mgr.AddNaviPosition('test5.py', 83)
        self.mgr.AddNaviPosition('test5.py', 45)
        self.mgr.AddNaviPosition('test6.py', 998)

        pos = self.mgr.GetPreviousNaviPos()
        self.assertEqual(pos[1], 998)

        pos = self.mgr.GetPreviousNaviPos()
        self.assertEqual(pos[1], 45)

        pos = self.mgr.GetPreviousNaviPos()
        self.assertEqual(pos[1], 83)



###############################################################################
# Name: _threads.py                                                           #
# Purpose: Threadpool implementation                                          #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2011 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""
Editra Business Model Library: ThreadPool


"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: _threads.py 67422 2011-04-09 17:23:27Z CJP $"
__revision__ = "$Revision: 67422 $"

__all__ = [ 'ThreadPool', ]

#-----------------------------------------------------------------------------#
# Imports
import threading
import Queue

#-----------------------------------------------------------------------------#

class ThreadPool(object):
    """Object for managing a collection of threads and dispatching jobs
    to them.

    """
    def __init__(self, tcount, qsize=-1):
        """Create the ThreadPool
        @param tcount: max number of threads to keep in the pool
        @keyword qsize: size of job queue (-1 for unlimited)

        """
        super(ThreadPool, self).__init__()

        # Attributes
        self._poolsize = tcount
        self._jobs = Queue.Queue(qsize)
        self._threads = [ _WorkerThread(self._jobs) for t in range(self._poolsize) ]

    ThreadCount = property(lambda self: self._poolsize)
    JobCount = property(lambda self: self._jobs.qsize())

    def QueueJob(self, funct, *args, **kwargs):
        """Add a job to be processed
        @param funct: callable
        @param args: list of any positional arguments to funct
        @param kwargs: map of any keyword arguments to funct

        """
        assert callable(funct)
        self._jobs.put((funct, args, kwargs))

    def Shutdown(self):
        """Shutdown the ThreadPool
        @note: Blocking call until all threads have exited

        """
        self._jobs.join()

#-----------------------------------------------------------------------------#

class _WorkerThread(threading.Thread):
    """Worker thread class to be used by the ThreadPool"""
    def __init__(self, jobs):
        """Create the Thread object
        @param jobs: Queue object

        """
        super(_WorkerThread, self).__init__()

        # Attributes
        self._jobs = jobs
        self.daemon = True
        self.start()

    def run(self):
        """Run and process jobs until requested to exit"""
        while True:
            funct, args, kwargs = self._jobs.get()
            try:
                funct(*args, **kwargs)
            except Exception, msg:
                pass # TODO add error to result data?
            finally:
                self._jobs.task_done()

#-----------------------------------------------------------------------------#
# Unittest
if __name__ == '__main__':
    pool = ThreadPool(5)
    import time
    import random
    def Job(id_, length):
        print "JOB: %d, begin" % id_
        time.sleep(length)
        print "JOB: %d, end" % id_

    print "Start Jobs"
    for x in range(8):
        pool.QueueJob(Job, x, random.randint(1, 20))
    print "All Jobs Queued"

    pool.Shutdown() # blocks till pool is shutdown
    print "All Done!"



###############################################################################
# Name: plugdlg.py                                                            #
# Purpose: User interface into the PluginManager, also provides interface for #
#          downloading and installing plugins.                                #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
Provides a dialog for downloading, installing and configuring plugins or Editra.

@todo: refactor list population and list item creation

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__cvsid__ = "$Id: plugdlg.py 70229 2012-01-01 01:27:10Z CJP $"
__revision__ = "$Revision: 70229 $"

#-----------------------------------------------------------------------------#
# Imports
import os
import sys
import re
import urllib2
import wx
import wx.lib.delayedresult as delayedresult

# Local Imports
import ed_glob
import plugin
import ed_event
import ed_msg
import util
import ed_txt
from profiler import Profile_Get, Profile_Set
import ed_basewin
import eclib

#-----------------------------------------------------------------------------#
# Globals

CONFIG_PG = 0
DOWNLOAD_PG = 1
INSTALL_PG = 2
PY_VER = str(sys.version_info[0]) + str(sys.version_info[1])
PLUGIN_REPO = "http://editra.org/plugins.php?list=True&py=" + PY_VER

# Panel Display Modes
MODE_CONFIG = 0
MODE_ERROR  = 1

# Image list indexes
IMG_CONFIG   = 0
IMG_DOWNLOAD = 1
IMG_INSTALL  = 2
IMG_ERROR    = 3
IMG_PLUGIN   = 4

_ = wx.GetTranslation

#-----------------------------------------------------------------------------#

def MakeThemeTool(tool_id):
    """Makes a themed bitmap for the tool book of the plugin dialog.
    @param tool_id: An art identifier id
    @return: 32x32 bitmap
    @todo: why does drawing a bitmap overlay on gtk not draw on transparent area

    """
    osize = Profile_Get('ICON_SZ', default=(24, 24))
    Profile_Set('ICON_SZ', (32, 32))
    base = wx.ArtProvider.GetBitmap(str(tool_id), wx.ART_TOOLBAR)
    Profile_Set('ICON_SZ', osize)
    if not base.IsOk():
        base = wx.ArtProvider.GetBitmap(wx.ART_WARNING,
                                        wx.ART_TOOLBAR,
                                        size=(32, 32))

    over = wx.ArtProvider.GetBitmap(str(ed_glob.ID_PLUGMGR), wx.ART_MENU)
    if over.IsOk():
        # Draw overlay onto button
        mdc = wx.MemoryDC()
        mdc.SelectObject(base)
        mdc.SetBrush(wx.TRANSPARENT_BRUSH)
        mdc.SetPen(wx.TRANSPARENT_PEN)
        mdc.DrawBitmap(over, 15, 15, False)
        mdc.SelectObject(wx.NullBitmap)

    return base

#-----------------------------------------------------------------------------#

class PluginDialog(wx.Frame):
    """Defines a Plugin manager Dialog that can be used to download plugins
    from a defined repository, offers services to install plugins that
    where downloaded with or without the dialog, as well as configure
    already installed plugins. It is instanciated as a standalone window
    when the show method is called so that if downloads are taking along time
    it does not interfere with usage of the editor.

    """
    def __init__(self, parent, id=wx.ID_ANY, title=u'', size=wx.DefaultSize):
        super(PluginDialog, self).__init__(parent, title=title, size=size,
                                           style=wx.DEFAULT_FRAME_STYLE)
        util.SetWindowIcon(self)

        # Attributes
        bstyle = eclib.SEGBOOK_STYLE_NO_DIVIDERS|eclib.SEGBOOK_STYLE_LABELS
        self._nb = eclib.SegmentBook(self, style=bstyle)
        self._cfg_pg = ConfigPanel(self._nb, style=wx.BORDER_SUNKEN)
        self._dl_pg = DownloadPanel(self._nb)
        self._inst_pg = InstallPanel(self._nb)
        self._imglst = list()

        # Setup
        self.__InitImageList()

        self._nb.AddPage(self._cfg_pg, _("Configure"), img_id=IMG_CONFIG)
        self._nb.AddPage(self._dl_pg, _("Download"), img_id=IMG_DOWNLOAD)
        self._nb.AddPage(self._inst_pg, _("Install"), img_id=IMG_INSTALL)

        # Check for plugins with error conditions and if any are found
        # Add the error page.
        pmgr = wx.GetApp().GetPluginManager()
        if len(pmgr.GetIncompatible()):
            self._nb.AddPage(ConfigPanel(self._nb, style=wx.NO_BORDER,
                                         mode=MODE_ERROR),
                             _("Errors"), img_id=IMG_ERROR)

        # Layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self._nb, 1, wx.EXPAND)
        self.SetSizer(sizer)
        self.SetStatusBar(eclib.ProgressStatusBar(self, style=wx.SB_FLAT))
        self.SetInitialSize(size)

        # Event Handlers
        self.Bind(eclib.EVT_SB_PAGE_CHANGING, self.OnPageChanging)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)

        # Message Handlers
        ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)

    def OnDestroy(self, evt):
        """Cleanup message handlers on delete"""
        if evt.GetId() == self.GetId():
            ed_msg.Unsubscribe(self.OnThemeChange)
        evt.Skip()

    def __InitImageList(self):
        """Initialize the segmentbooks image list"""
        dorefresh = False
        if len(self._imglst):
            del self._imglst
            self._imglst = list()
            dorefresh = True

        self._imglst.append(MakeThemeTool(ed_glob.ID_PREF))
        self._imglst.append(MakeThemeTool(ed_glob.ID_WEB))
        self._imglst.append(MakeThemeTool(ed_glob.ID_PACKAGE))
        bmp = wx.ArtProvider.GetBitmap(wx.ART_ERROR, wx.ART_TOOLBAR, (32, 32))
        self._imglst.append(bmp)
        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_PREF),
                                       wx.ART_TOOLBAR, (32, 32))
        self._imglst.append(bmp)
        self._nb.SetImageList(self._imglst)
        self._nb.SetUsePyImageList(True)

        if dorefresh:
            self._nb.Refresh()

    def OnClose(self, evt):
        """Handles closing the dialog and unregistering it from the mainloop.
        @param evt: wx.EVT_CLOSE

        """
        if self._dl_pg.IsDownloading():
            dlg = wx.MessageDialog(self, _("Downloads are incomplete"),
                                   _("Do you wish to exit?"),
                                   style=wx.YES_NO|wx.ICON_EXCLAMATION| \
                                           wx.CENTER)
            result = dlg.ShowModal()
            dlg.Destroy()
            if result == wx.ID_NO:
                return
            else:
                pass

        if self._cfg_pg.ConfigChanged():
            wx.MessageBox(_("You must restart Editra before your "
                            "changes will take full affect."),
                          _("Configuration Changes Made"),
                          wx.ICON_INFORMATION|wx.OK)
            

        wx.GetApp().UnRegisterWindow(repr(self))
        evt.Skip()

    def OnThemeChange(self, msg):
        """Update icons on theme change message"""
        self.__InitImageList()

    def Show(self, show=True):
        """Shows the dialog
        @postcondition: Dialog is registered with the main loop and shown

        """
        wx.GetApp().RegisterWindow(repr(self), self, True)
        wx.Frame.Show(self, show)

    def Busy(self, busy=True):
        """Set the status of the frame to be busy or not
        @keyword busy: Start or Stop being busy

        """
        if busy:
            self.GetStatusBar().StartBusy()
        else:
            self.GetStatusBar().StopBusy()

    def OnPageChanging(self, evt):
        """Updates pages as they are being changed to.
        @param evt: segmentbk.EVT_SB_PAGE_CHANGING

        """
        cur_pg = evt.GetSelection()
        self.SetTitle(self.GetTitle().split(" | ")[0] + \
                      " | " + self._nb.GetPageText(cur_pg))
        if cur_pg == CONFIG_PG:
            self._cfg_pg.PopulateCtrl()
            self.SetStatusText(_("Changes will take affect once the"
                                 " program has been restarted"), 0)
        elif cur_pg == DOWNLOAD_PG:
            self._dl_pg.UpdateList()
        elif cur_pg == INSTALL_PG:
            pass
        else:
            page = self._nb.GetPage(cur_pg)
            size = page.GetBestSize()
            s2 = self.GetSize()
            segbar = self._nb.GetSegmentBar()
            segsize = segbar.GetBestSize()
            self.SetClientSize((s2.GetWidth(),
                                size.GetHeight() + segsize.GetHeight()))
            self.SetStatusText("", 0)

        evt.Skip()

#-----------------------------------------------------------------------------#

class ConfigPanel(eclib.ControlBox):
    """Creates a panel for configuring plugins."""
    def __init__(self, parent, style=wx.NO_BORDER, mode=MODE_CONFIG):
        """Build config panel"""
        eclib.ControlBox.__init__(self, parent, style=style)

        # Attrtibutes
        self._mode = mode
        self._changed = False
        self._list = eclib.PanelBox(self)

        # Layout Panel
        self.SetWindow(self._list)

        if self._mode == MODE_CONFIG:
            self.PopulateCtrl()
        else:
            self.PopulateErrors()

        # Event Handlers
        self.Bind(ed_event.EVT_NOTIFY, self.OnNotify)

    def ConfigChanged(self):
        """Did the configuration change?
        @return: bool

        """
        return self._changed

    def GetItemIdentifier(self, name):
        """Gets the named item and returns its identifier. The
        identifier is the combination of the name and version
        strings.
        @param name: name of item in list
        @return: identifier for the named list item

        """
        identifer = None
        for item in self._list.GetItems():
            if item.GetPluginName().lower() == name.lower():
                identifer = (name, item.GetVersionString())
        return identifer

    def OnNotify(self, evt):
        """Handles the notification events that are
        posted from the list control.
        @param evt: ed_event.NotificationEvent

        """
        e_id = evt.GetId()
        if e_id == ed_glob.ID_PREF:
            # TODO: check for an open existing instance of this config objects
            #       page.
            cfg_obj = evt.GetValue()
            parent = self.GetParent() # SegmentBook

            bmp = cfg_obj.GetBitmap()
            if bmp.IsNull():
                idx = IMG_PLUGIN
            else:
                imglst = parent.GetImageList()
                imglst.append(bmp)
                idx = len(imglst) - 1

            label = cfg_obj.GetLabel()
            panel = cfg_obj.GetConfigPanel(parent)
            parent.AddPage(panel, label, True, idx)
            parent.SetSegmentCanClose(parent.GetPageCount() - 1, True)
        else:
            pname, enabled = evt.GetValue()
            pmgr = wx.GetApp().GetPluginManager()
            pmgr.EnablePlugin(pname, enabled)
            self._changed = True

    def PopulateCtrl(self):
        """Populates the list of plugins and sets the
        values of their states. Any successive calls to
        this function will clear the list and Repopulate it
        with current config values. Returns the number of
        items populated to the list
        @postcondition: list is populated with all plugins that are
                        currently loaded and sets the checkmarks accordingly
        @return: number of items added to list

        """
        p_mgr = wx.GetApp().GetPluginManager()
        if self._list.GetItemCount():
            self._list.DeleteAllItems()

        p_mgr.ReInit()
        config = p_mgr.GetConfig()
        keys = sorted([ ed_txt.DecodeString(name)
                        for name in config.keys() ],
                      key=unicode.lower)
        uninstalled = Profile_Get('UNINSTALL_PLUGINS', default=list())

        with eclib.Freezer(self._list) as _tmp:
            for item in keys:
                val = config[item]
                mod = sys.modules.get(item)
                dist = p_mgr.GetPluginDistro(item)
                if dist is not None:
                    item = dist.project_name
                    version = dist.version
                else:
                    version = str(getattr(mod, '__version__', _("Unknown")))

                pdata = PluginData()
                pdata.SetName(item)
                desc = getattr(mod, '__doc__', None)
                if not isinstance(desc, basestring):
                    desc = _("No Description Available")
                pdata.SetDescription(desc.strip())
                pdata.SetAuthor(getattr(mod, '__author__', _("Unknown")))
                pdata.SetVersion(version)
                pdata.SetDist(dist)
                pbi = PBPluginItem(self._list, mod, pdata, None)

                pbi.SetChecked(val)
                util.Log("[pluginmgr][info] Adding %s to list" % item)
                self._list.AppendItem(pbi)
                if pbi.GetInstallPath() in uninstalled:
                    pbi.Enable(False)

        self._list.SendSizeEvent()
        return self._list.GetItemCount()

    def PopulateErrors(self):
        """Populates the list of plugins and sets the
        values of their states. Any successive calls to
        this function will clear the list and Repopulate it
        with current config values. Returns the number of
        items populated to the list
        @postcondition: list is populated with all plugins that are
                        currently loaded and sets the checkmarks accordingly
        @return: number of items added to list

        """
        p_mgr = wx.GetApp().GetPluginManager()
        if self._list.GetItemCount():
            self._list.DeleteAllItems()

        p_mgr.ReInit()
        errors = p_mgr.GetIncompatible()
        keys = sorted([ ed_txt.DecodeString(name)
                        for name in errors.keys() ],
                      key=unicode.lower)
        bmp = wx.ArtProvider.GetBitmap(wx.ART_ERROR, wx.ART_TOOLBAR, (32, 32))
        msg = _("This plugin requires a newer version of Editra.")

        with eclib.Freezer(self._list) as _tmp:
            for item in keys:
                val = errors[item]
                mod = sys.modules.get(val)
                dist = p_mgr.GetPluginDistro(item)
                if dist is not None:
                    item = dist.project_name
                    version = dist.version
                else:
                    version = unicode(getattr(mod, '__version__', _("Unknown")))

                pin = PluginData()
                pin.SetName(item)
                pin.SetAuthor(getattr(mod, '__author__', _("Unknown")))
                pin.SetVersion(version)
                pin.SetDist(dist)
                pbi = PluginErrorItem(self._list, pin, msg, bmp=bmp)

                self._list.AppendItem(pbi)

        self._list.SendSizeEvent()
        return self._list.GetItemCount()

#-----------------------------------------------------------------------------#

class DownloadPanel(ed_basewin.EdBaseCtrlBox):
    """Creates a panel with controls for downloading plugins."""
    ID_DOWNLOAD = wx.NewId()
    EGG_PATTERN = re.compile(r"(?P<name>[^-]+)"
    r"( -(?P<ver>[^-]+) (-py(?P<pyver>[^-]+) (-(?P<plat>.+))? )? )?",
    re.VERBOSE | re.IGNORECASE
    ).match

    def __init__(self, parent):
        """Initializes the panel"""
        super(DownloadPanel, self).__init__(parent)

        # Attributes
        self._p_list = dict()           # list of available plugins/meta
        self._dl_list = dict()          # List of download urls
        self._eggcount = 0              # Number of plugins to download
        self._eggbasket = dict()        # Basket of downloaded eggs
        self._list = eclib.PanelBox(self)

        # Layout Panel
        cbar = self.CreateControlBar(wx.BOTTOM)
        cbar.AddStretchSpacer()
        self._downlb = wx.Button(cbar, DownloadPanel.ID_DOWNLOAD, _("Download"))
        self._downlb.Disable()
        cbar.AddControl(self._downlb, wx.ALIGN_RIGHT)
        self.SetWindow(self._list)

        # Event Handlers
        self.Bind(wx.EVT_BUTTON, self.OnButton)
        self.Bind(ed_event.EVT_NOTIFY, self.OnNotify)

    def _ResultCatcher(self, delayedResult):
        """Catches the results from the download worker threads"""
        # Check if result has come after the window is dead
        try:
            frame = self.GetGrandParent()
        except wx.PyDeadObjectError:
            return

        self._eggcount = self._eggcount - 1
        try:
            result = delayedResult.get()
            plug = result[0]
            if result[1]:
                self._eggbasket[plug] = result[2]
                frame.SetStatusText(_("Downloaded") + ": " + plug, 0)
        finally:
            if not self._eggcount:
                frame.SetStatusText(_("Finshed downloading plugins"), 0)
                wx.CallAfter(frame.Busy, False)
                inst_pg = self.GetParent().GetPage(INSTALL_PG)
                for key in self._eggbasket:
                    inst_pg.AddItemToInstall(key)
                self.GetParent().SetSelection(INSTALL_PG)

    def _UpdateCatcher(self, delayedResult):
        """Catches the results from the download worker threads"""
        try:
            frame = self.GetGrandParent()
            result = delayedResult.get()
            if len(result):
                self._p_list = self.FormatPluginList(result)
                self.PopulateList()
                frame.SetStatusText(_("Select plugins to download"), 0)
        except wx.PyDeadObjectError:
            return
        except Exception, msg:
            util.Log("[plugdlg][err] Download failed " + str(msg))
            frame.SetStatusText(_("Unable to retrieve plugin list"), 0)
        wx.CallAfter(frame.Busy, False)

    def FormatPluginList(self, data):
        """Formats a list of plugin data served by the server into
        PluginData objects for usage in the list view.
        @return: PluginData of all available plugins

        """
        plugins = data
        p_list = dict()
        if len(plugins) < 2:
            return p_list

        for meta in plugins:
            data = meta.split("\n")

            if len(data) < 4:
                continue

            tmpdat = PluginData()
            set_map = {'author' : tmpdat.SetAuthor,
                       'version' : tmpdat.SetVersion,
                       'name' : tmpdat.SetName,
                       'description' : tmpdat.SetDescription,
                       'url' : tmpdat.SetUrl}

            for attr in data:
                tmp = attr.split("=")
                if len(tmp) != 2:
                    continue
                
                funct = set_map.get(tmp[0].lower(), None)
                if funct:
                    funct(ed_txt.DecodeString(tmp[1].strip()))

            if tmpdat.GetName() != u'':
                p_list[ed_txt.DecodeString(tmpdat.GetName())] = tmpdat

        # Remove items that have already been installed
        config_pg = self.GetParent().GetPage(CONFIG_PG)
        to_clean = list()
        for pin in p_list:
            cfg_id = config_pg.GetItemIdentifier(pin.lower())
            if cfg_id is not None:
                try:
                    cur_id = [int(v) for v in cfg_id[1].split(".")]
                    dl_id = [int(v) for v in p_list[pin].GetVersion().split(".")]
                except:
                    continue
                if cur_id >= dl_id: # Installed version is >= avail dl
                    to_clean.append(pin)

        for item in to_clean:
            del p_list[item]

        return p_list

    def GetDownloadedData(self):
        """Returns the dictionary of downloaded data or an
        empty dictionary if no data has been downloaded.
        @return: set of all successfully downloaded plugins

        """
        return self._eggbasket

    def IsDownloading(self):
        """Returns whether the panel has active download
        threads or not.
        @return: bool

        """
        if self._eggcount:
            return True
        else:
            return False

    def OnButton(self, evt):
        """Handles the Button Events.
        @param evt: wx.EVT_BUTTON

        """
        e_id = evt.GetId()
        if e_id == self.ID_DOWNLOAD:
            urls = list()
            for item in self._dl_list:
                if self._dl_list[item] and item in self._p_list:
                    urls.append(self._p_list[item].GetUrl())
            self._eggcount = len(urls)

            # Start a separate thread to download each selection
            for egg in range(len(urls)):
                self.GetGrandParent().SetStatusText(_("Downloading") + "...", 0)
                self.GetGrandParent().Busy(True)
                delayedresult.startWorker(self._ResultCatcher, _DownloadPlugin,
                                          wargs=(urls[egg]), jobID=egg)
        else:
            evt.Skip()

    def OnNotify(self, evt):
        """Handles the notification events that are posted by the
        list control when items are checked.
        @param evt: ed_event.NotificationEvent

        """
        pin, enable = evt.GetValue()
        self._dl_list[pin] = enable

        if enable:
            self._downlb.Enable()
        else:
            for item in self._dl_list:
                if self._dl_list[item]:
                    self._downlb.Enable()
                    break
            else:
                self._downlb.Disable()

            if pin in self._dl_list:
                del self._dl_list[pin]

    def PopulateList(self):
        """Populates the list control based off data in the plugin data
        list. The plugin data list is set as a result of calling UpdateList
        it is not recommended to call this directly.

        @return: number of items added to control

        """
        if self._list.GetItemCount():
            self._list.DeleteAllItems()
        pins = sorted([ name for name in self._p_list.keys() ], key=unicode.lower)
        with eclib.Freezer(self) as _tmp:
            for item in pins:
                pbi = PBDownloadItem(self._list, self._p_list[item], None)
                self._list.AppendItem(pbi)
        self._list.SendSizeEvent()
        return self._list.GetItemCount()

    def RemoveDownloadedItem(self, item):
        """Remove an item from the download cache
        @param item: Name of item to remove

        """
        # Removed downloaded data
        if item in self._eggbasket:
            del self._eggbasket[item]

        # Remove download entry data
        match = self.EGG_PATTERN(item)
        if match:
            plugin_name = match.group('name').lower()
            if plugin_name in self._dl_list:
                del self._dl_list[plugin_name]

    def UpdateList(self, url=PLUGIN_REPO):
        """Update the list of available downloads
        @param url: url to fetch update list from
        @postcondition: Worker thread is started that will update list when it
                        finishes.

        """
        if self._list.GetItemCount():
            self._list.DeleteAllItems()
        frame =  self.GetGrandParent()
        frame.SetStatusText(_("Retrieving Plugin List") + "...", 0)
        frame.Busy(True)
        delayedresult.startWorker(self._UpdateCatcher, _GetPluginListData,
                                  wkwargs={'url' : url}, jobID='update')

#-----------------------------------------------------------------------------#
# Download utility functions

# The obtained meta data must be served as a file that is formatted
# as follows. Each meta data item must be on a single line with
# each set of meta data for different plugins separated by three
# hash marks '###'.
def _GetPluginListData(url=PLUGIN_REPO):
    """Gets the list of plugins and their related meta data
    as a string and returns it.
    @return: list of data of available plugins from website

    """
    text = u''
    try:
        try:
            if Profile_Get('USE_PROXY', default=False):
                proxy_set = Profile_Get('PROXY_SETTINGS',
                                        default=dict(uname='', url='',
                                                     port='80', passwd=''))
                proxy = util.GetProxyOpener(proxy_set)
                h_file = proxy.open(url)
            else:
                h_file = urllib2.urlopen(url)

            text = h_file.read()
            h_file.close()
        except (IOError, OSError), msg:
            util.Log("[plugdlg][err] %s" % str(msg))
    finally:
        return text.split("###")

def _DownloadPlugin(*args):
    """Downloads the plugin at the given url.
    @note: *args is really a string that has been exploded
    @return: name, completed, egg data
    @rtype: tuple

    """
    url = "".join(args)
    egg = None
    try:
        try:
            if Profile_Get('USE_PROXY', default=False):
                proxy_set = Profile_Get('PROXY_SETTINGS',
                                        default=dict(uname='', url='',
                                                     port='80', passwd=''))
                proxy = util.GetProxyOpener(proxy_set)
                h_file = proxy.open(url)
            else:
                h_file = urllib2.urlopen(url)

            egg = h_file.read()
            h_file.close()
        except (IOError, OSError), msg:
            util.Log("[plugdlg][err] %s" % str(msg))
    finally:
        return (url.split("/")[-1], True, egg)

#-----------------------------------------------------------------------------#

class InstallPanel(ed_basewin.EdBaseCtrlBox):
    """Creates a panel for installing plugins."""
    ID_INSTALL = wx.NewId()
    ID_USER = wx.NewId()
    ID_SYS = wx.NewId()
    ID_REMOVE_ITEM = wx.NewId()

    def __init__(self, parent):
        """Initializes the panel"""
        super(InstallPanel, self).__init__(parent)

        # Attributes
        bbar = self.CreateControlBar(wx.BOTTOM)
        toolt = wx.ToolTip(_("To add a new item drag and drop the plugin file "
                             "into the list.\n\nTo remove an item select it "
                             "and hit Delete or Backspace."))
        self._install = wx.ListBox(self, wx.ID_ANY,
                                   style=wx.LB_SORT|wx.BORDER_NONE)
        self._install.SetToolTip(toolt)
        self._install.SetDropTarget(util.DropTargetFT(self._install,
                                                      None, self.OnDrop))

        self._instb = wx.Button(bbar, self.ID_INSTALL, _("Install"))
        self._instb.Disable()
        self._usercb = wx.CheckBox(bbar, self.ID_USER, _("User Directory"))
        self._usercb.SetValue(True)
        toolt = wx.ToolTip(_("Install the plugins only for the current user"))
        self._usercb.SetToolTip(toolt)
        self._syscb = wx.CheckBox(bbar, self.ID_SYS, _("System Directory"))
        toolt = wx.ToolTip(_("Install the plugins for all users\n"
                             " **requires administrative privileges**"))
        self._syscb.SetToolTip(toolt)
        if not os.access(ed_glob.CONFIG['SYS_PLUGIN_DIR'], os.R_OK | os.W_OK):
            self._syscb.Disable()

        # Layout Panel
        self.SetWindow(self._install)
        bbar.AddControl(self._usercb)
        bbar.AddControl(self._syscb)
        bbar.AddStretchSpacer()
        bbar.AddControl(self._instb, wx.ALIGN_RIGHT)
        self.SendSizeEvent()

        # Event Handlers
        self.Bind(wx.EVT_BUTTON, self.OnButton)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox)
        self._install.Bind(wx.EVT_KEY_UP, self.OnKeyUp)

    def _Install(self):
        """Install the plugins in the list.
        @postcondition: all plugins listed in the list are installed and loaded

        """
        items = self._install.GetItems()
        inst_loc = ed_glob.CONFIG['PLUGIN_DIR']
        if self._syscb.GetValue():
            inst_loc = ed_glob.CONFIG['SYS_PLUGIN_DIR']

        for item in items:
            egg_name = item.split(os.sep)[-1]
            if os.path.isabs(item):
                try:
                    reader = file(item, "rb")
                    egg = reader.read()
                    reader.close()
                except (IOError, SystemError, OSError):
                    continue
            else:
                dl_pg = self.GetParent().GetPage(DOWNLOAD_PG)
                egg = dl_pg.GetDownloadedData().get(item, None)
                if not egg:
                    continue

            try:
                writer = file(inst_loc + egg_name, "wb")
                writer.write(egg)
                writer.close()
            except (IOError, OSError):
                continue
            else:
                # If successfully installed remove from list
                ind = self._install.FindString(item)
                dl_pg = self.GetParent().GetPage(DOWNLOAD_PG)
                if ind != wx.NOT_FOUND:
                    self._install.Delete(ind)
                    dl_pg.RemoveDownloadedItem(item)

        if not len(self._install.GetItems()):
            # All plugins installed correctly
            grand_p = self.GetTopLevelParent()
            grand_p.SetStatusText(_("Successfully Installed Plugins"), 0)
            # Note: need to do this because SetSelection doesn't fire a
            #       page change.
            wx.GetApp().GetPluginManager().ReInit()
            self.GetParent().SetSelection(CONFIG_PG)
            self._instb.Disable()
        else:
            self.GetGrandParent().SetStatusText(_("Error"), 1)
            dlg = wx.MessageDialog(self,
                                   _("Failed to install %d plugins") % \
                                   self._install.GetCount(),
                                   _("Installation Error"),
                                   style = wx.OK | wx.CENTER | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()

    def AddItemToInstall(self, item):
        """Adds an item to the install list, the item
        should be a string of the path to the item or
        the items name if it is an in memory file from the
        download page.
        @param item: path or name of plugin item

        """
        if self._install.FindString(item) == wx.NOT_FOUND:
            self._instb.Enable()
            self._install.Append(item)
        else:
            pass

    def OnButton(self, evt):
        """Handles button events generated by the panel.
        @param evt: wx.EVT_BUTTON

        """
        if evt.GetId() == self.ID_INSTALL:
            self._Install()
        else:
            evt.Skip()

    def OnCheckBox(self, evt):
        """Handles the checkbox events to make sure that
        only one of the two check boxes is checked at a time
        @param evt: wx.EVT_CHECKBOX

        """
        e_id = evt.GetId()
        val = evt.GetEventObject().GetValue()
        u_cb = self.FindWindowById(self.ID_USER)
        s_cb = self.FindWindowById(self.ID_SYS)
        if e_id == self.ID_USER:
            if not s_cb.IsEnabled():
                u_cb.SetValue(True)
            else:
                s_cb.SetValue(not val)
        elif e_id == self.ID_SYS:
            u_cb.SetValue(not val)
        else:
            pass
        evt.Skip()

    def OnDrop(self, files):
        """Get Drop files and place paths in control
        @todo: should also check entry points in addition to filetype
        @param files: list of file paths
        @postcondition: all non egg files are filtered only placing
                        the eggs in the list.
        """
        # Filter out any files that are not eggs
        good = [ fname for fname in files if fname.split(u'.')[-1] == u'egg' ]
        for item in good:
            if self._install.FindString(item) == wx.NOT_FOUND:
                self._install.Append(item)

        if self._install.GetCount():
            self._instb.Enable()

    def OnKeyUp(self, evt):
        """Key Event handler. Removes the selected item from
        the list control when the delete or backspace kis is pressed.
        @param evt: wx.KeyEvent(wx.EVT_KEY_UP)

        """
        if evt.GetKeyCode() in [wx.WXK_DELETE, wx.WXK_BACK]:
            item = self._install.GetSelection()
            if item != wx.NOT_FOUND:
                self._install.Delete(item)

            if not self._install.GetCount():
                self._instb.Disable()

        evt.Skip()

#-----------------------------------------------------------------------------#

class PBPluginItem(eclib.PanelBoxItemBase):
    """PanelBox Item to display configuration information about a plugin."""
    def __init__(self, parent, mod, pdata, bmp=None, enabled=False):
        """Create the PanelBoxItem
        @param parent: L{PanelBox}
        @param mod: module
        @param pdata: PluginData
        @keyword bmp: Plugin Icon
        @keyword enabled: Plugin is currently enabled (bool)

        """
        super(PBPluginItem, self).__init__(parent)

        # Attributes
        self._module = mod
        self._pdata = pdata
        self._bmp = bmp
        self._title = wx.StaticText(self, label=self._pdata.GetName())
        self._version = wx.StaticText(self, label=self._pdata.GetVersion())
        self._desc = wx.StaticText(self, label=self._pdata.GetDescription())
        self._auth = wx.StaticText(self, label=_("Author: %s") % self._pdata.GetAuthor())
        self._enabled = wx.CheckBox(self, label=_("Enable"))
        self._enabled.SetValue(enabled)
        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DELETE), wx.ART_MENU)
        self._uninstall = eclib.PlateButton(self, label=_("Uninstall"), bmp=bmp,
                                            style=eclib.PB_STYLE_NOBG)
        self._uninstall.Unbind(wx.EVT_ERASE_BACKGROUND)
        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_PREF), wx.ART_MENU)
        self._config = eclib.PlateButton(self,
                                         label=_("Configure"), bmp=bmp,
                                         style=eclib.PB_STYLE_NOBG)
        self._config.Unbind(wx.EVT_ERASE_BACKGROUND)
        self._config.Enable(enabled)

        # Setup
        if not hasattr(mod, 'GetConfigObject'):
            self._config.Hide()

        ipath = self.GetInstallPath()
        if ipath:
            if not os.access(ipath, os.R_OK|os.W_OK):
                self._uninstall.Show(False)
        else:
            util.Log("[pluginmgr][warn] cant find plugin path for %s" % \
                     self._pdata.GetName())
            self._uninstall.Show(False) # Should not happen

        font = self._title.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self._title.SetFont(font)
        self._version.SetFont(font)

        if wx.Platform == '__WXMAC__':
            for ctrl in (self._desc, self._auth, self._enabled,
                         self._config, self._uninstall):
                ctrl.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)

        # Layout
        self.__DoLayout()

        # Event Handlers
        self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
        self.Bind(wx.EVT_BUTTON, self.OnConfigButton, self._config)
        self.Bind(wx.EVT_BUTTON, self.OnUninstallButton, self._uninstall)

    def __DoLayout(self):
        """Layout the panel"""
        vsizer = wx.BoxSizer(wx.VERTICAL)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        # Left side Bitmap and Checkbox
        hsizer.Add((5, 5), 0)
        if self._bmp is not None:
            self._bmp = wx.StaticBitmap(self, bitmap=self._bmp)
            hsizer.Add(self._bmp, 0, wx.ALIGN_CENTER_VERTICAL)

        # Central area main content
        csizer = wx.BoxSizer(wx.VERTICAL)
        tsizer = wx.BoxSizer(wx.HORIZONTAL)
        tsizer.AddMany([(self._title, 0), ((20, -1), 1, wx.EXPAND),
                        (self._version, 0, wx.ALIGN_RIGHT),
                        ((5, 5), 0)])

        bsizer = wx.BoxSizer(wx.HORIZONTAL)
        bsizer.AddMany([(self._auth, 0), ((5, -1), 1, wx.EXPAND),
                        (self._enabled, 0, wx.ALIGN_CENTER_VERTICAL),
                        ((5, 5), 0),
                        (self._uninstall, 0, wx.ALIGN_CENTER_VERTICAL),
                        (self._config, 0, wx.ALIGN_CENTER_VERTICAL)])
        csizer.AddMany([(tsizer, 1, wx.EXPAND), ((3, 3), 0),
                        (self._desc, 0), ((3, 3), 0),
                        (bsizer, 0, wx.EXPAND)])

        # Finish Layout
        hsizer.AddMany([((5, 5), 0), (csizer, 1, wx.EXPAND), ((5, 5), 0)])
        vsizer.AddMany([((4, 4), 0), (hsizer, 0, wx.EXPAND), ((4, 4), 0)])
        self.SetSizer(vsizer)
        self.SetAutoLayout(True)

    def GetInstallPath(self):
        """Get the path of the plugin
        @return: string

        """
        if self._pdata is not None:
            dist = self._pdata.GetDist()
            if dist is not None:
                return dist.location
        return u''

    def GetPluginName(self):
        """Get the name of the plugin
        @return: string

        """
        return self._title.GetLabel()

    def GetVersionString(self):
        """Get the version of the plugin
        @return: string

        """
        return self._version.GetLabel()

    def OnConfigButton(self, evt):
        """Handle when the configuration button is hit."""
        if self._module is not None:
            cfg_obj = self._module.GetConfigObject()
            event = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY,
                                               ed_glob.ID_PREF, cfg_obj)
            wx.PostEvent(self.GetParent(), event)

    def OnUninstallButton(self, evt):
        """Uninstall the plugin"""
        msg = _("Are you sure you want to uninstall %s?\nThis cannot be undone.") 
        result = wx.MessageBox(msg % self.GetPluginName(),
                               _("Uninstall Plugin"),
                               wx.OK|wx.CANCEL|wx.ICON_WARNING)
        if result == wx.OK:
            self.Enable(False)
            self._desc.SetLabel(_("This plugin will be uninstalled on next program launch."))
            self._enabled.SetValue(False)
            pname = self._title.GetLabel()
            event = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY, self.GetId(),
                                               (pname, False), self)
            wx.PostEvent(self.GetParent(), event)
            plist = Profile_Get('UNINSTALL_PLUGINS', default=list())
            plist.append(self.GetInstallPath())
            Profile_Set('UNINSTALL_PLUGINS', plist)
        else:
            return

    def OnCheck(self, evt):
        """Notify container of changes to state of plugin"""
        enabled = self._enabled.GetValue()
        self._config.Enable(enabled)
        pname = self._title.GetLabel()
        event = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY, self.GetId(),
                                           (pname, enabled), self)
        wx.PostEvent(self.GetParent(), event)

    def SetChecked(self, check=True):
        """Set the checkbox
        @param check: bool

        """
        self._enabled.SetValue(check)
        self._config.Enable(check)

#-----------------------------------------------------------------------------#

class PBDownloadItem(PBPluginItem):
    """PanelBox Item to display download information about a plugin."""
    def __init__(self, parent, pdata, bmp=None):
        """Create the PanelBoxItem
        @param parent: L{PanelBox}
        @param pdata: PluginData
        @keyword bmp: Plugin Icon

        """
        super(PBDownloadItem, self).__init__(parent, None, pdata, bmp=bmp)

        # Setup
        self._uninstall.Hide()
        self._enabled.SetLabel(_("Download"))
        self.Layout()

#-----------------------------------------------------------------------------#

class PluginErrorItem(eclib.PanelBoxItemBase):
    """PanelBox Item to display configuration information about a plugin."""
    def __init__(self, parent, pdata, msg, bmp):
        """Create the PanelBoxItem
        @param parent: L{PanelBox}
        @param pdata: PluginData
        @param msg: error msg
        @param bmp: Bitmap

        """
        super(PluginErrorItem, self).__init__(parent)

        # Attributes
        self._bmp = bmp
        self._title = wx.StaticText(self, label=pdata.GetName())
        self._version = wx.StaticText(self, label=pdata.GetVersion())
        self._msg = wx.StaticText(self, label=msg)
        self._auth = wx.StaticText(self, label=_("Author: %s") % pdata.GetAuthor())

        # Setup
        font = self._title.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self._title.SetFont(font)
        self._version.SetFont(font)

        if wx.Platform == '__WXMAC__':
            self._msg.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
            self._auth.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)

        # Layout
        self.__DoLayout()

    def __DoLayout(self):
        """Layout the panel"""
        vsizer = wx.BoxSizer(wx.VERTICAL)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        # Left side Bitmap and Checkbox
        hsizer.Add((5, 5), 0)
        if self._bmp is not None:
            self._bmp = wx.StaticBitmap(self, bitmap=self._bmp)
            hsizer.Add(self._bmp, 0, wx.ALIGN_CENTER_VERTICAL)

        # Central area main content
        csizer = wx.BoxSizer(wx.VERTICAL)
        tsizer = wx.BoxSizer(wx.HORIZONTAL)
        tsizer.AddMany([(self._title, 0), ((20, -1), 1, wx.EXPAND),
                        (self._version, 0, wx.ALIGN_RIGHT)])

        bsizer = wx.BoxSizer(wx.HORIZONTAL)
        bsizer.AddMany([(self._auth, 0)])
        csizer.AddMany([(tsizer, 1, wx.EXPAND), ((3, 3), 0),
                        (self._msg, 0), ((3, 3), 0),
                        (bsizer, 0, wx.EXPAND)])

        # Finish Layout
        hsizer.AddMany([((5, 5), 0), (csizer, 1, wx.EXPAND), ((5, 5), 0)])
        vsizer.AddMany([((4, 4), 0), (hsizer, 0, wx.EXPAND), ((4, 4), 0)])
        self.SetSizer(vsizer)
        self.SetAutoLayout(True)

#-----------------------------------------------------------------------------#

class PluginData(plugin.PluginData):
    """Plugin Metadata storage class used to store data
    about plugins and where to download them from
    @see: plugin.PluginData

    """
    def __init__(self, name=u'', descript=u'', \
                 author=u'', ver=u'', url=u''):
        """Extends PluginData to include information about url to get it from.
        @keyword name: Plugin name
        @keyword descript: Plugin short description
        @keyword author: Plugin Author Name
        @keyword ver: Plugin Version (Unicode)
        @keyword url: url to download plugin from

        """
        super(PluginData, self).__init__(name, descript, author, ver)

        # Attributes
        self._url = url

    def GetUrl(self):
        """Returns the URL of the plugin
        @return: url string of plugins location

        """
        return self._url

    def SetUrl(self, url):
        """Sets the url of the plugin.
        @param url: fully qualified url string

        """
        if not isinstance(url, basestring):
            try:
                url = str(url)
            except (TypeError, ValueError):
                url = u''
        self._url = url

#-----------------------------------------------------------------------------#




###############################################################################
# Name: platebtn.py                                                           #
# Purpose: PlateButton is a flat label button with support for bitmaps and    #
#          drop menu.                                                         #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""
Editra Control Library: PlateButton

The PlateButton is a custom owner drawn flat button, that in many ways emulates
the buttons found the bookmark bar of the Safari browser. It can be used as a 
drop in replacement for wx.Button/wx.BitmapButton under most circumstances. It 
also offers a wide range of options for customizing its appearance, a 
description of each of the main style settings is listed below.

Main Button Styles:
Any combination of the following values may be passed to the constructor's style
keyword parameter.

PB_STYLE_DEFAULT:
Creates a flat label button with rounded corners, the highlight for mouse over
and press states is based off of the hightlight color from the systems current
theme.

PB_STYLE_GRADIENT:
The highlight and press states are drawn with gradient using the current
highlight color.

PB_STYLE_SQUARE:
Instead of the default rounded shape use a rectangular shaped button with
square edges.

PB_STYLE_NOBG:
This style only has an effect on Windows but does not cause harm to use on the
platforms. It should only be used when the control is shown on a panel or other
window that has a non solid color for a background. i.e a gradient or image is
painted on the background of the parent window. If used on a background with
a solid color it may cause the control to loose its transparent appearance.

PB_STYLE_DROPARROW:
Add a drop button arrow to the button that will send a separate event when
clicked on.

Other attributes can be configured after the control has been created. The
settings that are currently available are as follows:

  - SetBitmap: Change/Add the bitmap at any time and the control will resize and
               refresh to display it.
  - SetLabelColor: Explicitly set text colors
  - SetMenu: Set the button to have a popupmenu. When a menu is set a small drop
             arrow will be drawn on the button that can then be clicked to show
             a menu.
  - SetPressColor: Use a custom highlight color


Overridden Methods Inherited from PyControl:

  - SetFont: Changing the font is one way to set the size of the button, by
             default the control will inherit its font from its parent.

  - SetWindowVariant: Setting the window variant will cause the control to
                      resize to the corresponding variant size. However if the
                      button is using a bitmap the bitmap will remain unchanged
                      and only the font will be adjusted.

Requirements:
  - python2.4 or higher
  - wxPython2.8 or higher

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: platebtn.py 73153 2012-12-08 20:23:44Z CJP $"
__revision__ = "$Revision: 73153 $"

__all__ = ["PlateButton",
           "PLATE_NORMAL", "PLATE_PRESSED", "PLATE_HIGHLIGHT", 

           "PB_STYLE_DEFAULT", "PB_STYLE_GRADIENT", "PB_STYLE_SQUARE",
           "PB_STYLE_NOBG", "PB_STYLE_DROPARROW", "PB_STYLE_TOGGLE",

           "EVT_PLATEBTN_DROPARROW_PRESSED"]

#-----------------------------------------------------------------------------#
# Imports
import wx
import wx.lib.newevent

# Local Imports
from eclutil import *

#-----------------------------------------------------------------------------#
# Button States
PLATE_NORMAL = 0
PLATE_PRESSED = 1
PLATE_HIGHLIGHT = 2

# Button Styles
PB_STYLE_DEFAULT  = 1   # Normal Flat Background
PB_STYLE_GRADIENT = 2   # Gradient Filled Background
PB_STYLE_SQUARE   = 4   # Use square corners instead of rounded
PB_STYLE_NOBG     = 8   # Usefull on Windows to get a transparent appearance
                        # when the control is shown on a non solid background
PB_STYLE_DROPARROW = 16 # Draw drop arrow and fire EVT_PLATEBTN_DROPRROW_PRESSED event
PB_STYLE_TOGGLE   = 32  # Stay pressed untill clicked again

#-----------------------------------------------------------------------------#

# EVT_BUTTON used for normal event notification
# EVT_TOGGLE_BUTTON used for toggle button mode notification
PlateBtnDropArrowPressed, EVT_PLATEBTN_DROPARROW_PRESSED = wx.lib.newevent.NewEvent()

#-----------------------------------------------------------------------------#

class PlateButton(wx.PyControl):
    """PlateButton is a custom type of flat button with support for
    displaying bitmaps and having an attached dropdown menu.

    """
    def __init__(self, parent, id=wx.ID_ANY, label='', bmp=None, 
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=PB_STYLE_DEFAULT, name=wx.ButtonNameStr):
        """Create a PlateButton
        @keyword label: Buttons label text
        @keyword bmp: Buttons bitmap
        @keyword style: Button style

        """
        super(PlateButton, self).__init__(parent, id, pos, size,
                                          wx.BORDER_NONE|wx.TRANSPARENT_WINDOW,
                                          name=name)

        # Attributes
        self.InheritAttributes()
        self._bmp = dict(enable=None, disable=None)
        if bmp is not None:
            assert isinstance(bmp, wx.Bitmap) and bmp.IsOk()
            self._bmp['enable'] = bmp
            img = bmp.ConvertToImage()
            img = img.ConvertToGreyscale(.795, .073, .026) #(.634, .224, .143)
            self._bmp['disable'] = wx.BitmapFromImage(img)

        self._menu = None
        self.SetLabel(label)
        self._style = style
        self._state = dict(pre=PLATE_NORMAL, cur=PLATE_NORMAL)
        self._color = self.__InitColors()
        self._pressed = False

        # Setup Initial Size
        self.SetInitialSize(size)

        # Event Handlers
        self.Bind(wx.EVT_PAINT, lambda evt: self.__DrawButton())
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnErase)
        self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
        self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)

        # Mouse Events
        self.Bind(wx.EVT_LEFT_DCLICK, lambda evt: self._ToggleState())
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_ENTER_WINDOW,
                  lambda evt: self._SetState(PLATE_HIGHLIGHT))
        self.Bind(wx.EVT_LEAVE_WINDOW,
                  lambda evt: wx.CallLater(80, self.__LeaveWindow))

        # Other events
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        self.Bind(wx.EVT_CONTEXT_MENU, lambda evt: self.ShowMenu())

    def __DrawBitmap(self, gc):
        """Draw the bitmap if one has been set
        @param gc: GCDC to draw with
        @return: x cordinate to draw text at

        """
        if self.IsEnabled():
            bmp = self._bmp['enable']
        else:
            bmp = self._bmp['disable']

        if bmp is not None and bmp.IsOk():
            bw, bh = bmp.GetSize()
            ypos = (self.GetSize()[1] - bh) // 2
            gc.DrawBitmap(bmp, 6, ypos, bmp.GetMask() != None)
            return bw + 6
        else:
            return 6

    def __DrawDropArrow(self, gc, xpos, ypos):
        """Draw a drop arrow if needed and restore pen/brush after finished
        @param gc: GCDC to draw with
        @param xpos: x cord to start at
        @param ypos: y cord to start at

        """
        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
            # Positioning needs a little help on Windows
            if wx.Platform == '__WXMSW__':
                xpos -= 2
            tripoints = [(xpos, ypos), (xpos + 6, ypos), (xpos + 3, ypos + 5)]
            brush_b = gc.GetBrush()
            pen_b = gc.GetPen()
            gc.SetPen(wx.TRANSPARENT_PEN)
            gc.SetBrush(wx.Brush(gc.GetTextForeground()))
            gc.DrawPolygon(tripoints)
            gc.SetBrush(brush_b)
            gc.SetPen(pen_b)
        else:
            pass

    def __DrawHighlight(self, gc, width, height):
        """Draw the main highlight/pressed state
        @param gc: GCDC to draw with
        @param width: width of highlight
        @param height: height of highlight

        """
        if self._state['cur'] == PLATE_PRESSED:
            color = self._color['press']
        else:
            color = self._color['hlight']

        if self._style & PB_STYLE_SQUARE:
            rad = 0
        else:
            rad = (height - 3) / 2

        if self._style & PB_STYLE_GRADIENT:
            gc.SetBrush(wx.TRANSPARENT_BRUSH)
            rgc = gc.GetGraphicsContext()
            brush = rgc.CreateLinearGradientBrush(0, 1, 0, height,
                                                  color, AdjustAlpha(color, 55))
            rgc.SetBrush(brush)
        else:
            gc.SetBrush(wx.Brush(color))

        gc.DrawRoundedRectangle(1, 1, width - 2, height - 2, rad)

    def __PostEvent(self):
        """Post a button event to parent of this control"""
        if self._style & PB_STYLE_TOGGLE:
            etype = wx.wxEVT_COMMAND_TOGGLEBUTTON_CLICKED
        else:
            etype = wx.wxEVT_COMMAND_BUTTON_CLICKED
        bevt = wx.CommandEvent(etype, self.GetId())
        bevt.SetEventObject(self)
        bevt.SetString(self.GetLabel())
        self.GetEventHandler().ProcessEvent(bevt)

    def __DrawButton(self):
        """Draw the button"""
        dc = wx.PaintDC(self)
        gc = wx.GCDC(dc)

        # Setup
        dc.SetBrush(wx.TRANSPARENT_BRUSH)
        gc.SetBrush(wx.TRANSPARENT_BRUSH)
        gc.SetFont(self.Font)
        dc.SetFont(self.Font)
        gc.SetBackgroundMode(wx.TRANSPARENT)

        # The background needs some help to look transparent on
        # on Gtk and Windows
        if wx.Platform in ['__WXGTK__', '__WXMSW__']:
            gc.SetBackground(self.GetBackgroundBrush(gc))
            gc.Clear()

        # Calc Object Positions
        width, height = self.GetSize()
        if wx.Platform == '__WXGTK__':
            tw, th = dc.GetTextExtent(self.Label)
        else:
            tw, th = gc.GetTextExtent(self.Label)
        txt_y = max((height - th) // 2, 1)

        if self._state['cur'] == PLATE_HIGHLIGHT:
            gc.SetTextForeground(self._color['htxt'])
            gc.SetPen(wx.TRANSPARENT_PEN)
            self.__DrawHighlight(gc, width, height)

        elif self._state['cur'] == PLATE_PRESSED:
            gc.SetTextForeground(self._color['htxt'])
            if wx.Platform == '__WXMAC__':
                pen = wx.Pen(GetHighlightColour(), 1, wx.SOLID)
            else:
                pen = wx.Pen(AdjustColour(self._color['press'], -80, 220), 1)
            gc.SetPen(pen)

            self.__DrawHighlight(gc, width, height)
            txt_x = self.__DrawBitmap(gc)
            if wx.Platform == '__WXGTK__':
                dc.DrawText(self.Label, txt_x + 2, txt_y)
            else:
                gc.DrawText(self.Label, txt_x + 2, txt_y)
            self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)

        else:
            if self.IsEnabled():
                gc.SetTextForeground(self.GetForegroundColour())
            else:
                txt_c = wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT)
                gc.SetTextForeground(txt_c)

        # Draw bitmap and text
        if self._state['cur'] != PLATE_PRESSED:
            txt_x = self.__DrawBitmap(gc)
            if wx.Platform == '__WXGTK__':
                dc.DrawText(self.Label, txt_x + 2, txt_y)
            else:
                gc.DrawText(self.Label, txt_x + 2, txt_y)
            self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)

    def __InitColors(self):
        """Initialize the default colors"""
        color = GetHighlightColour()
        pcolor = AdjustColour(color, -12)
        colors = dict(default=True,
                      hlight=color, 
                      press=pcolor,
                      htxt=BestLabelColour(self.GetForegroundColour()))
        return colors

    def __LeaveWindow(self):
        """Handle updating the buttons state when the mouse cursor leaves"""
        if (self._style & PB_STYLE_TOGGLE) and self._pressed:
            self._SetState(PLATE_PRESSED) 
        else:
            self._SetState(PLATE_NORMAL)
            self._pressed = False

    def _SetState(self, state):
        """Manually set the state of the button
        @param state: one of the PLATE_* values
        @note: the state may be altered by mouse actions
        @note: Internal use only!

        """
        self._state['pre'] = self._state['cur']
        self._state['cur'] = state
        if wx.Platform == '__WXMSW__':
            self.Parent.RefreshRect(self.Rect, False)
        else:
            self.Refresh()

    def _ToggleState(self):
        """Toggle button state
        @note: Internal Use Only!

        """
        if self._state['cur'] != PLATE_PRESSED:
            self._SetState(PLATE_PRESSED)
        else:
            self._SetState(PLATE_HIGHLIGHT)

    #---- End Private Member Function ----#

    #---- Public Member Functions ----#

    BitmapDisabled = property(lambda self: self.GetBitmapDisabled(),
                              lambda self, bmp: self.SetBitmapDisabled(bmp))
    BitmapLabel = property(lambda self: self.GetBitmapLabel(),
                           lambda self, bmp: self.SetBitmap(bmp))

    # Aliases
    BitmapFocus = BitmapLabel
    BitmapHover = BitmapLabel
    BitmapSelected = BitmapLabel

    LabelText = property(lambda self: self.GetLabel(),
                         lambda self, lbl: self.SetLabel(lbl))

    def AcceptsFocus(self):
        """Can this window have the focus?"""
        return self.IsEnabled()

    def Disable(self):
        """Disable the control"""
        super(PlateButton, self).Disable()
        self.Refresh()

    def DoGetBestSize(self):
        """Calculate the best size of the button
        @return: wx.Size

        """
        width = 4
        height = 6
        if self.Label:
            # NOTE: Should measure with a GraphicsContext to get right
            #       size, but due to random segfaults on linux special
            #       handling is done in the drawing instead...
            lsize = self.GetFullTextExtent(self.Label)
            width += lsize[0]
            height += lsize[1]
            
        if self._bmp['enable'] is not None:
            bsize = self._bmp['enable'].Size
            width += (bsize[0] + 10)
            if height <= bsize[1]:
                height = bsize[1] + 6
            else:
                height += 3
        else:
            width += 10

        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
            width += 12

        best = wx.Size(width, height)
        self.CacheBestSize(best)
        return best

    def Enable(self, enable=True):
        """Enable/Disable the control"""
        super(PlateButton, self).Enable(enable)
        self.Refresh()

    def GetBackgroundBrush(self, dc):
        """Get the brush for drawing the background of the button
        @return: wx.Brush
        @note: used internally when on gtk

        """
        if wx.Platform == '__WXMAC__' or self._style & PB_STYLE_NOBG:
            return wx.TRANSPARENT_BRUSH

        bkgrd = self.GetBackgroundColour()
        brush = wx.Brush(bkgrd, wx.SOLID)
        my_attr = self.GetDefaultAttributes()
        p_attr = self.Parent.GetDefaultAttributes()
        my_def = bkgrd == my_attr.colBg
        p_def = self.Parent.GetBackgroundColour() == p_attr.colBg
        if my_def and not p_def:
            bkgrd = self.Parent.GetBackgroundColour()
            brush = wx.Brush(bkgrd, wx.SOLID)
        return brush

    def GetBitmapDisabled(self):
        """Get the bitmap of the disable state
        @return: wx.Bitmap or None

        """
        return self.BitmapDisabled

    def GetBitmapLabel(self):
        """Get the label bitmap
        @return: wx.Bitmap or None

        """
        return self.BitmapLabel

    # GetBitmap Aliases for BitmapButton api
    GetBitmapFocus = GetBitmapLabel
    GetBitmapHover = GetBitmapLabel
    
    # Alias for GetLabel
    GetLabelText = wx.PyControl.GetLabel

    def GetMenu(self):
        """Return the menu associated with this button or None if no
        menu is associated with it.

        """
        return self._menu

    def GetState(self):
        """Get the current state of the button
        @return: int
        @see: PLATE_NORMAL, PLATE_HIGHLIGHT, PLATE_PRESSED

        """
        return self._state['cur']

    def HasTransparentBackground(self):
        """Override setting of background fill"""
        return True

    def IsPressed(self):
        """Return if button is pressed (PB_STYLE_TOGGLE)
        @return: bool

        """
        return self._pressed

    #---- Event Handlers ----#

    def OnErase(self, evt):
        """Trap the erase event to keep the background transparent
        on windows.
        @param evt: wx.EVT_ERASE_BACKGROUND

        """
        if not (PB_STYLE_NOBG & self._style):
            evt.Skip()

    def OnFocus(self, evt):
        """Set the visual focus state if need be"""
        if self._state['cur'] == PLATE_NORMAL:
            self._SetState(PLATE_HIGHLIGHT)

    def OnKeyUp(self, evt):
        """Execute a single button press action when the Return key is pressed
        and this control has the focus.
        @param evt: wx.EVT_KEY_UP

        """
        if evt.GetKeyCode() == wx.WXK_SPACE:
            self._SetState(PLATE_PRESSED)
            self.__PostEvent()
            wx.CallLater(100, self._SetState, PLATE_HIGHLIGHT)
        else:
            evt.Skip()

    def OnKillFocus(self, evt):
        """Set the visual state back to normal when focus is lost
        unless the control is currently in a pressed state.

        """
        # Note: this delay needs to be at least as much as the on in the KeyUp
        #       handler to prevent ghost highlighting from happening when
        #       quickly changing focus and activating buttons
        if self._state['cur'] != PLATE_PRESSED:
            self._SetState(PLATE_NORMAL)

    def OnLeftDown(self, evt):
        """Sets the pressed state and depending on the click position will
        show the popup menu if one has been set.

        """
        if (self._style & PB_STYLE_TOGGLE):
            self._pressed = not self._pressed

        pos = evt.GetPositionTuple()
        self._SetState(PLATE_PRESSED)
        size = self.GetSizeTuple()
        if pos[0] >= size[0] - 16:
            if self._menu is not None:
                self.ShowMenu()
            elif self._style & PB_STYLE_DROPARROW:
                event = PlateBtnDropArrowPressed()
                event.SetEventObject(self)
                self.EventHandler.ProcessEvent(event)
        
        self.SetFocus()

    def OnLeftUp(self, evt):
        """Post a button event if the control was previously in a
        pressed state.
        @param evt: wx.MouseEvent

        """
        if self._state['cur'] == PLATE_PRESSED:
            pos = evt.GetPositionTuple()
            size = self.GetSizeTuple()
            if not (self._style & PB_STYLE_DROPARROW and pos[0] >= size[0] - 16):
                self.__PostEvent()

        if self._pressed:
            self._SetState(PLATE_PRESSED)
        else:
            self._SetState(PLATE_HIGHLIGHT)

    def OnMenuClose(self, evt):
        """Refresh the control to a proper state after the menu has been
        dismissed.
        @param evt: wx.EVT_MENU_CLOSE

        """
        mpos = wx.GetMousePosition()
        if self.HitTest(self.ScreenToClient(mpos)) != wx.HT_WINDOW_OUTSIDE:
            self._SetState(PLATE_HIGHLIGHT)
        else:
            self._SetState(PLATE_NORMAL)
        evt.Skip()

    #---- End Event Handlers ----#

    def SetBitmap(self, bmp):
        """Set the bitmap displayed in the button
        @param bmp: wx.Bitmap

        """
        self._bmp['enable'] = bmp
        img = bmp.ConvertToImage()
        img = img.ConvertToGreyscale(.795, .073, .026) #(.634, .224, .143)
        self._bmp['disable'] = img.ConvertToBitmap()
        self.InvalidateBestSize()

    def SetBitmapDisabled(self, bmp):
        """Set the bitmap for the disabled state
        @param bmp: wx.Bitmap

        """
        self._bmp['disable'] = bmp

    # Aliases for SetBitmap* functions from BitmapButton
    SetBitmapFocus = SetBitmap
    SetBitmapHover = SetBitmap
    SetBitmapLabel = SetBitmap
    SetBitmapSelected = SetBitmap

    def SetFocus(self):
        """Set this control to have the focus"""
        if self._state['cur'] != PLATE_PRESSED:
            self._SetState(PLATE_HIGHLIGHT)
        super(PlateButton, self).SetFocus()

    def SetFont(self, font):
        """Adjust size of control when font changes"""
        super(PlateButton, self).SetFont(font)
        self.InvalidateBestSize()

    def SetLabel(self, label):
        """Set the label of the button
        @param label: lable string

        """
        super(PlateButton, self).SetLabel(label)
        self.InvalidateBestSize()

    def SetLabelColor(self, normal, hlight=wx.NullColour):
        """Set the color of the label. The optimal label color is usually
        automatically selected depending on the button color. In some
        cases the colors that are chosen may not be optimal.
        
        The normal state must be specified, if the other two params are left
        Null they will be automatically guessed based on the normal color. To
        prevent this automatic color choices from happening either specify
        a color or None for the other params.

        @param normal: Label color for normal state (wx.Colour)
        @keyword hlight: Color for when mouse is hovering over

        """
        assert isinstance(normal, wx.Colour), "Must supply a colour object"
        self._color['default'] = False
        self.SetForegroundColour(normal)

        if hlight is not None:
            if hlight.IsOk():
                self._color['htxt'] = hlight
            else:
                self._color['htxt'] = BestLabelColour(normal)

        if wx.Platform == '__WXMSW__':
            self.Parent.RefreshRect(self.GetRect(), False)
        else:
            self.Refresh()

    def SetMenu(self, menu):
        """Set the menu that can be shown when clicking on the
        drop arrow of the button.
        @param menu: wxMenu to use as a PopupMenu
        @note: Arrow is not drawn unless a menu is set

        """
        if self._menu is not None:
            self.Unbind(wx.EVT_MENU_CLOSE)

        self._menu = menu
        self.Bind(wx.EVT_MENU_CLOSE, self.OnMenuClose)
        self.InvalidateBestSize()

    def SetPressColor(self, color):
        """Set the color used for highlighting the pressed state
        @param color: wx.Colour
        @note: also resets all text colours as necessary

        """
        self._color['default'] = False
        if color.Alpha() == 255:
            self._color['hlight'] = AdjustAlpha(color, 200)
        else:
            self._color['hlight'] = color
        self._color['press'] = AdjustColour(color, -10, 160)
        self._color['htxt'] = BestLabelColour(self._color['hlight'])
        self.Refresh()

    def SetWindowStyle(self, style):
        """Sets the window style bytes, the updates take place
        immediately no need to call refresh afterwards.
        @param style: bitmask of PB_STYLE_* values

        """
        self._style = style
        self.Refresh()

    def SetWindowVariant(self, variant):
        """Set the variant/font size of this control"""
        super(PlateButton, self).SetWindowVariant(variant)
        self.InvalidateBestSize()

    def ShouldInheritColours(self):
        """Overridden base class virtual. If the parent has non-default
        colours then we want this control to inherit them.

        """
        return True

    def ShowMenu(self):
        """Show the dropdown menu if one is associated with this control"""
        if self._menu is not None:
            size = self.GetSizeTuple()
            adj = wx.Platform == '__WXMAC__' and 3 or 0

            if self._style & PB_STYLE_SQUARE:
                xpos = 1
            else:
                xpos = size[1] / 2

            self.PopupMenu(self._menu, (xpos, size[1] + adj))

    #---- End Public Member Functions ----#



###############################################################################
# Name: haxe.py                                                               #
# Purpose: Syntax Definitions for haXe web language                           #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
@summary: Lexer configuration module for haXe web programming language

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: _haxe.py 68798 2011-08-20 17:17:05Z CJP $"
__revision__ = "$Revision: 68798 $"

#-----------------------------------------------------------------------------#
# Imports
import wx.stc as stc

# Local Imports
import synglob
import syndata
import _cpp

#-----------------------------------------------------------------------------#

#---- Keyword Definitions ----#
HAXE_KW = (0, "abstract break case catch class const continue trace do else "
              "enum extends finally for function goto if implements import in "
              "instanceof int interface new package private public return "
              "static super switch this throw throws transient try typeof var "
              "void volatile while with" )

HAXE_TYPES = (1, "Bool Enum false Float Int null String true Void ")

#---- End Keyword Definitions ----#

#-----------------------------------------------------------------------------#

class SyntaxData(syndata.SyntaxDataBase):
    """SyntaxData object for HaXe""" 
    def __init__(self, langid):
        super(SyntaxData, self).__init__(langid)

        # Setup
        self.SetLexer(stc.STC_LEX_CPP)
        self.RegisterFeature(synglob.FEATURE_AUTOINDENT, _cpp.AutoIndenter)

    def GetKeywords(self):
        """Returns Specified Keywords List """
        return [HAXE_KW, HAXE_TYPES, _cpp.DOC_KEYWORDS]

    def GetSyntaxSpec(self):
        """Syntax Specifications """
        return _cpp.SYNTAX_ITEMS

    def GetProperties(self):
        """Returns a list of Extra Properties to set """
        return [_cpp.FOLD,]

    def GetCommentPattern(self):
        """Returns a list of characters used to comment a block of code """
        return ['//']



###############################################################################
# Name: ed_mdlg.py                                                            #
# Purpose: Commonly used message dialogs                                      #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
# License: wxWindows License                                                  #
###############################################################################

"""
This module provides a number of message dialogs that are commonly used
throughout Editra. Its purpose is to promote reuse of the common dialogs for
consistancy and reduction in redundant code.

@summary: Common dialogs and related convenience functions

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: ed_mdlg.py 66817 2011-01-29 21:32:20Z CJP $"
__revision__ = "$Revision: 66817 $"

#--------------------------------------------------------------------------#
# Imports
import wx
import wx.stc
from extern.embeddedimage import PyEmbeddedImage

# Editra Library
import ed_glob
import util
import eclib

#--------------------------------------------------------------------------#
# Globals

_ = wx.GetTranslation

FileIcon = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAABthJ"
    "REFUWIW9l21sVFkZgJ/7Pd8zLdOWQloiIA2WbiqBLWhhdw3IAhuyRqMxq64uv1B/GMJqYvzK"
    "qqkmfqx/3UiC7q4QN5o1UnbRPyVLgdYtCAjdju2Uwhbabmd6O9PpdO7H8Ud7h2lnaLv+8E1u"
    "zj3vnXPe57znfd9zRqJcIseOHXsxGo1W5fP5nOu6xQ+OIwAXT+e6LrZtUygUdE3T3O7u7pP9"
    "/f03K8y5egkGgy2maTpiBXEcR1iWJXK5nBgbGxPJZFJ0d3eL3bt3H/4w9qSlinA43NbT03O5"
    "oaGBdDoNgBAC13WLreM4xb4QAtu2SaVSzMzMEI/HOX78+KGLFy+eWw2AXEYkSWKhLdWhKAqy"
    "LCPLMoqiFPuSJKFpGrFYjImJCUzTpLOzs7O1tfXg/wTgGaz0eIZVVS2+eyC6rhOLxRgaGiIS"
    "idDV1dXZ0tKyIkQZgGVZopJxb/XeqksBvCcQCCCEIJPJEIlEuHLlSueOHTuWhajogVJjnkG3"
    "JA48AE3Tit7wtsHv99Pf38/g4CCWZXH27NnOtra2R0KoSxXV1dWKruvFrXDd+bRzhUC47iJv"
    "2LZDKGigqgqTqSy1tbVks1nOnDnDyMgIwWCQXbt20dHR0XnixImn+/r63l5qrywLDh48uP/0"
    "6dPnFUXBNE0cy0Lz+9Grq3EdB5jPfyc7Q33cRyo1zcR4hnhjPXmhUJjLk0gkGLpxA3t2ltm5"
    "ORobGzEMI3306NHHUqnUvWU9sHXr1ng4HMY0TRRFwRcIYOdymAMDyJqGNZmm1nCItGzm0nWT"
    "F37Yx8c32Jz8js34/TkwdOK2Q9W2baiBAIV8nkwmQ01NTVVTU9OnL126dHJZgLVr18a91DIM"
    "A7/fz8TwMOaXn6chGmBNewsDH32Cb/xxlOvvJWleU8vVbD2dL/+Zw9fOM2FaCOHi/OznRJub"
    "sSYmiMViyLJMLpfzrxgDmqb5AAzDQFEUAHyFAi2BAsqhw5xSW3n5wizbxACnmsdpbdV4IxNk"
    "w2QM4wOTUP8gbjhM1tBxFgqVYRgEAgE0TVtqrhzAsqwcgKIoxYj3r1vLXz73I875d3H15k1+"
    "teMuTwUNHiR0JmerOLAlTu+4Rr69HXfGxhEOuqZh6Dr5hSzy+/0YhlEWc2UAyWTyfXhYjKYn"
    "U3z/lb9zJRVAQqLev4XaDQ5EFLJOlM0HdnI7rfLcrx/Q9ewetoyNku4fJuTzEfL7wedDCIGq"
    "qchyedaXabq7uycymUyxPxeuYn+Dj4vSGxwI/pO3bmn8picMbU1sfuEQd2b8dLzyHx70K7yU"
    "qIP9e1nf+jFq6msxAJ/Ph67rqIpK6cn6SIBkMlnI5/MAFCyLGl2ifUcz6X/0ccT3Lvvb5kik"
    "6/nbhTR/Opei7bnXyZq3ee17Phx5kluBOq637OHUhQQaYPh8xYIFiBW3AJA8V3kb5kQi3Pv8"
    "19i+r4Uv3XufjrONvPhbhTX2X3n1x4+z75Nb4NYgz1h3MXqv8qrSzC97E3zxQDPBUDXZhQJW"
    "Sco8oKqqJMnzP/ZAFKDRdWBgki80zrK+apzEgxDPf7aVffubYFzCHpki2NWLoZnkwptI3A0x"
    "en9s0TyVYqDMA7ZtC89RHrWwHXJ3htHyc4RrdL7ZrnAnHeP1y2v5RPRdmqU8qgY8+yl+/2+D"
    "H/TYfGWPReO6mkXzrMoDpeIFjSRc3A8mcadSzF4e4EhdhiNtGW6PxXjtXzroM1ybinKgt56X"
    "+mf5ae0Ffnd8O1owTi6XWxagUgwgxOJYEbYNd+8iWRZzcwX87wi++pEC4ztruJbaxTPnrzI2"
    "PcxeaZQ3Iwl8l3sxx48SqlvsyVUBWJZVBChts/k8SiaDpRuEJoM0PxnDvHqf0fvDtFfd5CfG"
    "NVpHhsjcGGFQ1YjrKhEe1hOgWFlX9IAnkiThAqFNm1j/1jkkSSJSFeK9xCjf+sXbhKI+/vDt"
    "x2nZ+BnE0JOkbBc34KdOUQisW4dtO4sAVuWBpeLaNqphEN24sagbJc2e9ga++/XDoEQQgPtY"
    "I1EPHLALBWyrgFR+4q8M4BF7rXcT9t73bt/EUzu3AGDbNm5Jnns3ZSHmxwtAkh4d66sCmL+O"
    "C2D+WlawCsj24vshzOe5Bzs/VEIIgbxQV7xFfGiA+VYsTCYX/x94xh+CLh7vSaUCVPz2yC9L"
    "JvBWWwq5VCfLi2/SlWCWSpkHVFWVFg6ORYMrXSaWg60kmqatfB+wbduZmpoiHA4zPT1d1Jf+"
    "PxBCIFyBK9zyolXS9941TSMUClEoFMrO40r+qQ6FQk/Islznuq5NyREaCARkwzBk27ZFPp93"
    "LcsqO14fIaokSblMJvMOkFzlmP+P/BeZah5l10evBAAAAABJRU5ErkJggg==")

#--------------------------------------------------------------------------#

def OpenErrorDlg(parent, fname, err):
    """Show a file open error dialog
    @param parent: parent window
    @param fname: file that failed to open
    @param err: error message

    """
    argmap = dict(filename=fname, errormsg=err)
    dlg = wx.MessageDialog(parent,
                           _("Editra could not open %(filename)s\n\n"
                             "Error:\n%(errormsg)s") % \
                           argmap, _("Error Opening File"),
                           style=wx.OK|wx.CENTER|wx.ICON_ERROR)
    dlg.CenterOnParent()
    result = dlg.ShowModal()
    dlg.Destroy()
    return result

def SaveErrorDlg(parent, fname, err):
    """Show a file save error modal dialog
    @param parent: window that the dialog is the child of
    @param fname: name of file that error occured
    @param err: the err message/description
    @return: wxID_OK if dialog was shown and dismissed properly

    """
    argmap = dict(filename=fname, errormsg=err)
    dlg = wx.MessageDialog(parent,
                           _("Failed to save file: %(filename)s\n\n"
                             "Error:\n%(errormsg)s") % argmap,
                           _("Save Error"), wx.OK|wx.ICON_ERROR)
    dlg.CenterOnParent()
    result = dlg.ShowModal()
    dlg.Destroy()
    return result

#--------------------------------------------------------------------------#

class EdFileInfoDlg(eclib.FileInfoDlg):
    """File information dialog"""
    def __init__(self, parent, fname):
        """General file information dialog
        @param parent: parent window
        @param fname: file path

        """
        super(EdFileInfoDlg, self).__init__(parent, fname=fname, ftype=None, 
                                            bmp=FileIcon.GetBitmap())

        # Setup
        self.SetFileTypeLabel(util.GetFileType(fname))

#--------------------------------------------------------------------------#

class EdFormatEOLDlg(eclib.ChoiceDialog):
    """Dialog for selecting EOL format"""
    def __init__(self, parent, msg=u'', title=u'', selection=0):
        """Create the dialog
        @keyword selection: default selection (wx.stc.STC_EOL_*)

        """
        choices = [_("Old Machintosh (\\r)"), _("Unix (\\n)"),
                   _("Windows (\\r\\n)")]
        self._eol = [wx.stc.STC_EOL_CR, wx.stc.STC_EOL_LF, wx.stc.STC_EOL_CRLF]
        idx = self._eol.index(selection)
        super(EdFormatEOLDlg, self).__init__(parent, msg=msg, title=title,
                                             choices=choices,
                                             style=wx.YES_NO|wx.YES_DEFAULT)
        self.SetSelection(idx)

        # Setup
        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DOCPROP), wx.ART_OTHER)
        if bmp.IsOk():
            self.SetBitmap(bmp)
        self.CenterOnParent()

    def GetSelection(self):
        """Get the selected eol mode
        @return: wx.stc.STC_EOL_*

        """
        sel = super(EdFormatEOLDlg, self).GetSelection()
        return self._eol[sel]



###############################################################################
# Name: AuiPaneNavigator.py                                                   #
# Purpose: Test and demo file for eclib.AuiPaneNavigator                      #
# Author: Cody Precord <cprecord@editra.org>                                  #
# Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
# Licence: wxWindows Licence                                                  #
###############################################################################

"""
Test file for testing the AuiPaneNavigator

"""

__author__ = "Cody Precord <cprecord@editra.org>"
__svnid__ = "$Id: AuiNavigatorDemo.py 61841 2009-09-06 14:11:00Z CJP $"
__revision__ = "$Revision: 61841 $"

#-----------------------------------------------------------------------------#
# Imports
import os
import sys
import wx
import wx.aui as aui

import IconFile

# Put local package on the path
#sys.path.insert(0, os.path.abspath('../../src'))
import eclib

#-----------------------------------------------------------------------------#
ID_NAVI = wx.NewId()

#-----------------------------------------------------------------------------#

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        wx.Panel.__init__(self, parent, wx.ID_ANY, size=(500,500))

        # Attributes
        self._paneNavi = None
        self.mgr = aui.AuiManager(self, aui.AUI_MGR_ALLOW_ACTIVE_PANE|aui.AUI_MGR_TRANSPARENT_HINT)
        self.log = log

        # Setup
        panes = list()
        for pane in range(4):
            tmp = wx.Panel(self, size=(100,100))
            panes.append(tmp)

        if wx.Platform != '__WXMAC__':
            txt = wx.StaticText(panes[0], label="Ctrl+1 to activate navigator")
        else:
            txt = wx.StaticText(panes[0], label="Alt+Tab to activate navigator")
        sizer = wx.BoxSizer()
        sizer.Add(txt, 0, wx.ALIGN_CENTER|wx.ALL, 20)
        panes[0].SetSizer(sizer)

        # Populate aui manager
        name = "Panel%s"
        pos = ['Center', 'Left', 'Top', 'Right']
        for idx, pane in enumerate(panes):
            tmp = name % idx
            if idx > 0:
                txt = wx.TextCtrl(pane)
                s = wx.BoxSizer()
                s.Add(txt, 1, wx.EXPAND)
                pane.SetSizer(s)

            ai = aui.AuiPaneInfo().Caption(tmp).MinSize((100,100)).Name(tmp)
            ai = getattr(ai, pos[idx])()
            self.mgr.AddPane(pane, ai)
        self.mgr.Update()

        tlw = self.GetTopLevelParent()
        if wx.Platform != '__WXMAC__':
            tbl = wx.AcceleratorTable([(wx.ACCEL_CTRL, ord('1'), ID_NAVI)])
        else:
            tbl = wx.AcceleratorTable([(wx.ACCEL_ALT, wx.WXK_TAB, ID_NAVI)])
        tlw.SetAcceleratorTable(tbl)
        tlw.Bind(wx.EVT_MENU, self.OnNavigate, id=ID_NAVI)

        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy)

    def OnDestroy(self, evt):
        # DEMO hack unbind event handler on delete
        tlw = self.GetTopLevelParent()
        if tlw is not None:
            tlw.Unbind(wx.EVT_MENU, tlw, id=ID_NAVI)
            tlw.SetAcceleratorTable(wx.AcceleratorTable([]))

    def OnNavigate(self, evt):
        """Make use of the AuiPaneNavigator"""
        if self._paneNavi is not None:
            evt.Skip()
            return

        self._paneNavi = eclib.AuiPaneNavigator(self,
                                                self.mgr,
                                                IconFile.Address.GetBitmap(),
                                                "Navigator Test")
        self._paneNavi.SetReturnCode(wx.ID_OK)
        self._paneNavi.ShowModal()

        sel = self._paneNavi.GetSelection()
        self._paneNavi.Destroy()
        self._paneNavi = None

        if isinstance(sel, basestring):
            paneInfo = self.mgr.GetPane(sel)
            if paneInfo.IsOk():
                if not paneInfo.IsShown():
                    paneInfo.Show()
                    self.mgr.Update()
                paneInfo.window.SetFocus()

#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

class TestLog:
    def __init__(self):
        pass

    def write(self, msg):
        print msg

#----------------------------------------------------------------------

overview = eclib.auinavi.__doc__
title = "AuiPaneNavigator"

#-----------------------------------------------------------------------------#
if __name__ == '__main__':
    try:
        import run
    except ImportError:
        app = wx.PySimpleApp(False)
        frame = wx.Frame(None, title="AuiPaneNavigator Demo")
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(TestPanel(frame, TestLog()), 1, wx.EXPAND)
        frame.CreateStatusBar()
        frame.SetSizer(sizer)
        frame.SetInitialSize((300, 300))
        frame.Show()
        app.MainLoop()
    else:
        run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])
